# -*- coding: utf-8 -*-

import base64
import phonenumbers
from django.core import validators
from django.core.validators import validate_email
from django.core.exceptions import ValidationError
from rest_framework import serializers
from clariweb.clariweb.models import Document, ClariwebUser


__all__ = [
    'UserAddSerializer', 'DocumentSerializer', 'UserMergeSerializer',
    'DocumentWriteSerializer', 'UserWriteSerializer',
]


MAPPING_TYPES_UTILISATEUR = {
    1: "PAT",
    2: "MED",
    3: 'COR',
    4: "PREL",
}

MAPPING_CONTENT_TYPE = {
    "application/pdf": 'PDF'
}


class Base64Field(serializers.Field):
    def to_internal_value(self, value):
        if value in validators.EMPTY_VALUES or value is None:
            return None
        return base64.b64decode(value.encode("ascii"))

    def to_representation(self, value):
        assert(False)


class PhoneField(serializers.Field):
    def to_internal_value(self, v):
        textnumber = None
        if v:
            try:
                pnumber = phonenumbers.parse(v, 'FR')
            except phonenumbers.phonenumberutil.NumberParseException:
                pass
            else:
                if phonenumbers.is_valid_number_for_region(
                        pnumber, "FR"):
                    textnumber = phonenumbers.format_number(
                        pnumber,
                        phonenumbers.PhoneNumberFormat.E164)
        return textnumber


class OnlyValidEmailField(serializers.CharField):
    def to_internal_value(self, value):
        if not value:
            return ''
        try:
            validate_email(value)
        except ValidationError:
            return ''
        else:
            return value


class NullStringDateField(serializers.DateField):
    def to_internal_value(self, value):
        if value is None or len(value) == 0:
            return None
        return super().to_internal_value(value)


class ActorSerializer(serializers.Serializer):
    nom = serializers.CharField(required=True, max_length=256)
    prenom = serializers.CharField(required=False, allow_null=True,
                                   allow_blank=True, max_length=256)
    civilite = serializers.CharField(required=False, allow_null=True,
                                     allow_blank=True, max_length=128)
    date_naissance = NullStringDateField(
        required=False, allow_null=True,
        input_formats=["%Y-%m-%dT%H:%M:%SZ"])


class PersonneSerializer(ActorSerializer):
    numero_secu = serializers.CharField(required=False, allow_null=True,
                                        allow_blank=True, max_length=32)


class AdresseSerializer(serializers.Serializer):
    ville = serializers.CharField(required=False, allow_null=True,
                                  allow_blank=True, max_length=256)
    code_postal = serializers.CharField(required=False, allow_null=True,
                                        allow_blank=True, max_length=16)
    ligne1 = serializers.CharField(required=False, allow_null=True,
                                   allow_blank=True, max_length=256)
    ligne2 = serializers.CharField(required=False, allow_null=True,
                                   allow_blank=True, max_length=256)
    ligne3 = serializers.CharField(required=False, allow_null=True,
                                   allow_blank=True, max_length=256)
    telephone = PhoneField(required=False, allow_null=True)
    gsm = PhoneField(required=False, allow_null=True)
    email = OnlyValidEmailField(required=False, allow_null=True,
                                allow_blank=True, max_length=1024)


class DemandeUnpaidSerializer(serializers.Serializer):
    demande_id = serializers.IntegerField(required=True)
    amount = serializers.IntegerField(required=True)


class UserMoreSerializer(serializers.Serializer):
    groupe = serializers.CharField(required=False, allow_null=True,
                                   allow_blank=True, max_length=1024)
    groupe_clinique = serializers.CharField(required=False, allow_null=True,
                                            allow_blank=True, max_length=1024)
    service_clinique = serializers.CharField(required=False, allow_null=True,
                                             allow_blank=True, max_length=1024)
    force_password = serializers.BooleanField(default=False)
    debpt = serializers.IntegerField(default=0)
    demandes_unpaid = serializers.ListField(child=DemandeUnpaidSerializer(),
                                            required=False)


class DemandeSerializer(serializers.Serializer):
    id_demande = serializers.IntegerField(required=True)
    numero_demande = serializers.CharField(max_length=128, required=True)
    date_demande = serializers.DateTimeField(
        input_formats=["%Y-%m-%dT%H:%M:%SZ"], required=True)


class FileSerializer(serializers.Serializer):
    type_document = serializers.CharField(required=True, max_length=128)
    content_type = serializers.CharField(required=True, max_length=128)
    static_file = Base64Field(required=False, allow_null=True)
    facture = serializers.BooleanField(default=False)
    invoice_id = serializers.CharField(required=False, max_length=32,
                                       allow_null=True, allow_blank=True)

    def validate_content_type(self, value):
        try:
            return MAPPING_CONTENT_TYPE[value]
        except KeyError:
            raise serializers.ValidationError(
                "content_type {} invalid.".format(value))
        return value


class AnalyseSerializer(serializers.Serializer):
    code = serializers.CharField(required=True)
    libelle = serializers.CharField(required=True)
    anormaux = serializers.ListField(
        child=serializers.CharField(required=False))
    a_saisir = serializers.BooleanField()


class MoreDemandeSerializer(serializers.Serializer):
    type_notification = serializers.IntegerField(default=0)
    analyses = serializers.ListField(child=AnalyseSerializer(required=False),
                                     required=False)
    debpt = serializers.IntegerField(default=0)
    structured_results = serializers.JSONField(required=False)


class ClinicDemandeSerializer(serializers.Serializer):
    bed = serializers.CharField(
        max_length=128, required=False, allow_null=True, allow_blank=True)
    ipp = serializers.CharField(
        max_length=128, required=False, allow_null=True, allow_blank=True)
    hospi = serializers.CharField(
        max_length=128, required=False, allow_null=True, allow_blank=True)


class UserAddSerializer(serializers.Serializer):
    id_utilisateur = serializers.IntegerField(required=True)
    type = serializers.IntegerField(required=True)
    login = serializers.CharField(required=True, max_length=128)
    password = serializers.CharField(required=True)
    personne = PersonneSerializer()
    adresse = AdresseSerializer()
    more = UserMoreSerializer()

    def validate_type(self, value):
        try:
            return MAPPING_TYPES_UTILISATEUR[value]
        except KeyError:
            raise serializers.ValidationError("type {} invalid.".format(value))
        return value


class DocumentSerializer(serializers.Serializer):
    id_document = serializers.IntegerField(required=False, allow_null=True)
    id_utilisateur = serializers.IntegerField(required=True)
    patient = ActorSerializer()
    medecins = ActorSerializer(many=True, required=False)
    transmetteur = ActorSerializer(required=False)
    demande = DemandeSerializer()
    file = FileSerializer()
    more = MoreDemandeSerializer()
    clinic = ClinicDemandeSerializer(required=False)


class DocumentWriteSerializer(serializers.ModelSerializer):
    class Meta:
        model = Document
        exclude = ('structured_results', )
    analyses = serializers.ListField(child=serializers.DictField(),
                                     required=False)


class DocumentWriteSerializerStructured(DocumentWriteSerializer):
    class Meta:
        model = Document
        exclude = ('datafile', 'sil_document_pk')


class UserMergeSerializer(serializers.Serializer):
    id_utilisateur_dest = serializers.IntegerField(required=True)
    id_utilisateurs_source = serializers.ListField(
        child=serializers.IntegerField(required=True))


class UserWriteSerializer(serializers.ModelSerializer):
    class Meta:
        model = ClariwebUser
        fields = "__all__"
