# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import os
import six
import shutil
import datetime
import constance
from django.core.cache import cache
from django.core import mail
from rest_framework import status
from rest_framework.test import APITestCase, APIClient
from django.urls import reverse
from django.utils import timezone
from django.conf import settings
from simple_email_confirmation.models import EmailAddress
from clariweb.clariweb.models import (
    BasicUser,
    ClariwebUser,
    Document,
    DocumentAccessLog,
    AccessLog,
    Notification,
    PasswordReset,
)
from clariweb.clariweb.smscount import SMSCounter


class EmptyCacheAPITestCase(APITestCase):
    def tearDown(self):
        super().tearDown()
        cache.clear()


class AdminTestCase(EmptyCacheAPITestCase):
    def setUp(self):
        super().setUp()
        constance.config.SECURITY_IP_ADDRESSES_ADMIN = ["127.0.0.0/8"]
        self.client = self.client_class(REMOTE_ADDR="127.0.0.1")


class BasicTestCase(object):
    def _test_login(self, username, shouldpass=True, **kwargs):
        data = {"username": username,
                "password": kwargs.get("password", username)}
        url = reverse("api-login")
        response = self.client.post(
            url, data, REMOTE_ADDR=kwargs.get("REMOTE_ADDR", "127.0.0.1"))
        if shouldpass:
            assert response.status_code == status.HTTP_200_OK
            assert response.data["isLogged"] is True
            reponseuser = response.data["user"]
            assert reponseuser["username"] == username
            assert reponseuser["first_name"] == kwargs.get('first_name', username)
            assert reponseuser["last_name"] == kwargs.get('last_name', username)
            assert reponseuser["email"] == kwargs.get('email', '')
            self.client.credentials(
                HTTP_X_SESSION_ID=response.data["sessionid"])
        return response

    def createClariwebUser(self, username, kind, sil_pk, **kwargs):
        user = BasicUser.objects.create_user(**{
            "username": username,
            "password": username,
            "first_name": kwargs.get("first_name", username),
            "last_name": kwargs.get("last_name", username),
            "email": kwargs.get("email"),
        })
        clariwebuser = ClariwebUser(**{
            "sil_pk": sil_pk,
            "kind": kind,
            "group": kwargs.get("group", ""),
            "civilite": kwargs.get("civilite", "Mr"),
            "date_naissance": kwargs.get("date_naissance"),
            "address_mobile_phone": kwargs.get("address_mobile_phone", ""),
            "debpt": kwargs.get("debpt", 0),
        })
        if kwargs.get("numero_secu"):
            clariwebuser.numero_secu = kwargs["numero_secu"]
        if kwargs.get("service_clinique"):
            clariwebuser.service_clinic = kwargs["service_clinique"]
        if kwargs.get("group_clinic"):
            clariwebuser.group_clinic = kwargs["group_clinic"]
        clariwebuser.save()
        user.clariwebuser = clariwebuser
        user.save()
        return user

    def createDocument(self, user, patient, demande_number, **kwargs):
        self.sil_next_pk = getattr(self, "sil_next_pk", 0) + 1
        fpath = kwargs.get("fpath", None)
        if fpath is None:
            fpath = self.test_fpath
        doc = Document.objects.create(**{
            "sil_document_pk": self.sil_next_pk,
            "sil_demande_pk": int(demande_number),
            "sil_demande_number": str(demande_number),
            "sil_demande_date": kwargs.get("sil_demande_date", timezone.now()),
            "user": user.clariwebuser,
            "datafile": fpath,
            "patient": patient,
            "document": kwargs.get("document", "Compte-Rendu PDF"),
            "medecin": kwargs.get("medecin", None),
            "correspondant": kwargs.get("correspondant", None),
            "notify_sms": kwargs.get("notify_sms", False),
            "notify_email": kwargs.get("notify_email", False),
        })
        return doc


class LoginTestCase(EmptyCacheAPITestCase, BasicTestCase):
    def setUp(self):
        super().setUp()
        self.su = BasicUser.objects.create_superuser(
            username="root", password="root")
        self.nobody = BasicUser.objects.create_user(
            username="nobody", password="nobody", email="nobody@nobody.com")
        self.inactive = BasicUser.objects.create_user(
            username="inactive", password="inactive")
        self.inactive.is_active = False
        self.inactive.save()

    def test_login_su(self):
        constance.config.SECURITY_IP_ADDRESSES_ADMIN = ["127.0.0.0/8"]
        self._test_login("root", first_name='', last_name='')

    def test_login_su_badip(self):
        constance.config.SECURITY_IP_ADDRESSES_ADMIN = ["8.8.8.8/32"]
        response = self._test_login(
            "root", first_name='', last_name='', shouldpass=False)
        assert response.status_code == status.HTTP_401_UNAUTHORIZED

    def test_login_nobody(self):
        self._test_login(
            "nobody", first_name='', last_name='', email="nobody@nobody.com")

    def test_badlogin(self):
        response = self._test_login("root", password="pouet", shouldpass=False)
        assert response.status_code == status.HTTP_401_UNAUTHORIZED
        assert response.data["isLogged"] is False
        assert response.data["active"] is False

    def test_bad_data(self):
        r = self.client.post(reverse("api-login"), {})
        assert r.status_code == status.HTTP_400_BAD_REQUEST
        assert "errors" in r.data

    def test_inactive(self):
        response = self._test_login("inactive", shouldpass=False)
        assert response.status_code == status.HTTP_401_UNAUTHORIZED
        assert response.data["isLogged"] is False
        assert response.data["active"] is False

    def test_logout(self):
        self._test_login(
            "nobody", first_name='', last_name='', email='nobody@nobody.com')
        logout_r = self.client.get(reverse('api-logout'))
        assert logout_r.status_code == status.HTTP_200_OK
        response = self.client.get(reverse('document-listall'))
        assert response.status_code == status.HTTP_403_FORBIDDEN

    def test_session_expiry_loggedout(self):
        url = reverse("api-login-setsessionexpiry")
        response = self.client.post(url)
        assert response.status_code == status.HTTP_403_FORBIDDEN

    def test_session_expiry_patient(self):
        self._test_login(
            "nobody", first_name='', last_name='', email='nobody@nobody.com')
        url = reverse("api-login-setsessionexpiry")
        for expiry in settings.SESSION_LOGIN_EXPIRY_CHOICES.keys():
            response = self.client.post(url, {"expiry_choice": expiry})
            if expiry == 1:
                assert response.status_code == status.HTTP_200_OK
            else:
                assert response.status_code == status.HTTP_403_FORBIDDEN

    def test_session_expiry_patient_10minutes(self):
        self._test_login(
            "nobody", first_name='', last_name='', email='nobody@nobody.com')
        url = reverse("api-login-setsessionexpiry")
        response = self.client.post(url, {'expiry_choice': 1})
        assert response.status_code == status.HTTP_200_OK
        assert response.data["expiry_time"] == settings.SESSION_LOGIN_EXPIRY_CHOICES[1]  # noqa

    def test_session_expiry_medecin_disabled(self):
        med1 = self.createClariwebUser(
            "med1", "MED", 10, civilite="Dr",
            first_name="laurent", last_name="COUSTET")
        self._test_login("med1", first_name="laurent", last_name="COUSTET")
        constance.config.LOGIN_USING_SESSIONS = False
        assert not med1.is_using_session_login()
        url = reverse("api-login-setsessionexpiry")
        response = self.client.post(url, {'expiry_choice': 2})
        assert response.status_code == status.HTTP_403_FORBIDDEN

    def test_session_expiry_medecin_enabled(self):
        med1 = self.createClariwebUser(
            "med1", "MED", 10, civilite="Dr",
            first_name="laurent", last_name="COUSTET")
        self._test_login("med1", first_name="laurent", last_name="COUSTET")
        constance.config.LOGIN_USING_SESSIONS = True
        assert med1.is_using_session_login()
        url = reverse("api-login-setsessionexpiry")
        for expiry in settings.SESSION_LOGIN_EXPIRY_CHOICES.keys():
            response = self.client.post(url, {"expiry_choice": expiry})
            assert response.status_code == status.HTTP_200_OK

    def test_session_expiry_medecin_invalid_expiry(self):
        med1 = self.createClariwebUser(
            "med1", "MED", 10, civilite="Dr",
            first_name="laurent", last_name="COUSTET")
        self._test_login("med1", first_name="laurent", last_name="COUSTET")
        constance.config.LOGIN_USING_SESSIONS = True
        assert med1.is_using_session_login()
        url = reverse("api-login-setsessionexpiry")
        expiry = 400
        assert expiry not in settings.SESSION_LOGIN_EXPIRY_CHOICES.keys()
        response = self.client.post(url, {"expiry_choice": expiry})
        assert response.status_code == status.HTTP_400_BAD_REQUEST

    def test_login_using_sessions_fail(self):
        url = reverse("api-login")
        response = self.client.get(url)
        assert response.status_code == status.HTTP_401_UNAUTHORIZED

    def test_login_using_sessions_bad_session_id(self):
        url = reverse("api-login")
        response = self.client.get(url, **{"X-SESSION-ID": "blabla"})
        assert response.status_code == status.HTTP_401_UNAUTHORIZED

    def test_login_data_debpt_below_disabled(self):
        pat = self.createClariwebUser("pat", "PAT", 10, civilite="Mr")
        constance.config.DEBPT_MINIMUM_AMOUNT = 0
        pat.clariwebuser.debpt = 500
        pat.clariwebuser.save()
        response = self._test_login("pat", first_name="pat", last_name="pat")
        assert response.data["user"]["debpt"] == 500
        assert not response.data["user"]["debpt_overlimit"]

    def test_login_data_debpt_below_minimum(self):
        pat = self.createClariwebUser("pat", "PAT", 10, civilite="Mr")
        constance.config.DEBPT_MINIMUM_AMOUNT = 100
        pat.clariwebuser.debpt = 10
        pat.clariwebuser.save()
        response = self._test_login("pat", first_name="pat", last_name="pat")
        assert response.data["user"]["debpt"] == 10
        assert not response.data["user"]["debpt_overlimit"]

    def test_login_data_debpt_above_minimum(self):
        pat = self.createClariwebUser("pat", "PAT", 10, civilite="Mr")
        constance.config.DEBPT_MINIMUM_AMOUNT = 100
        pat.clariwebuser.debpt = 500
        pat.clariwebuser.save()
        response = self._test_login("pat", first_name="pat", last_name="pat")
        assert response.data["user"]["debpt"] == 500
        assert response.data["user"]["debpt_overlimit"]

    def test_login_data_noclariwebuser_debpt(self):
        response = self._test_login(
            "nobody", first_name='', last_name='', email='nobody@nobody.com')
        assert response.data["user"]["debpt"] == 0

    def test_login_patient_underaged(self):
        constance.config.UNDERAGE_FORBIDDEN = True
        assert constance.config.UNDERAGE_FORBIDDEN
        self.createClariwebUser("underaged", "PAT", 10, civilite="Enf",
                                date_naissance=datetime.date(2003, 1, 1))
        response = self._test_login("underaged", shouldpass=False)
        assert response.status_code == status.HTTP_403_FORBIDDEN

    def test_login_patient_underaged_settings_disabled(self):
        constance.config.UNDERAGE_FORBIDDEN = False
        assert not constance.config.UNDERAGE_FORBIDDEN
        self.createClariwebUser("underaged", "PAT", 10, civilite="Enf",
                                date_naissance=datetime.date(2003, 1, 1))
        response = self._test_login("underaged")
        assert response.status_code == status.HTTP_200_OK

    def test_login_uppercase_username(self):
        r = self.client.post(reverse('api-login'), data={
            "username": "NoBoDy", "password": "nobody"})
        assert r.status_code == 200, r

    def test_login_accented_username(self):
        r = self.client.post(reverse('api-login'), data={
            "username": "NoBôDy", "password": "nobody"})
        assert r.status_code == 200, r

    def test_login_email(self):
        r = self.client.post(reverse('api-login'), data={
            "username": "NoBoDy@nobody.com", "password": "nobody"})
        assert r.status_code == 200, r

    def test_login_upperemail(self):
        BasicUser.objects.create_user(
            username="UpperEmail", password="upemail",
            email="UpperEmail@Test.com")
        r = self.client.post(reverse('api-login'), data={
            "username": "upperemail@test.com", "password": "upemail"})
        assert r.status_code == 200, r

    def test_email_overwrite_existing_user(self):
        pat1 = self.createClariwebUser("pat1", "PAT", 30,
                                       email="pat1@analyses.fr")
        pat2 = self.createClariwebUser("pat2", "PAT", 31, civilite="Mme")
        pat1.add_unconfirmed_email(pat1.email)
        assert not pat1.is_confirmed
        pat1.confirm_email(pat1.confirmation_key, "pat1@analyses.fr")
        assert pat1.is_confirmed
        pat2.email = "pat1@analyses.fr"
        pat2.save()
        pat2.confirm_email(pat2.confirmation_key, "pat1@analyses.fr")
        assert pat1.is_confirmed
        assert pat2.is_confirmed
        # Test login using username and email on pat1
        for login in (pat1.username, pat1.email):
            r = self.client.post(reverse('api-login'),
                                 data={"username": login, "password": "pat1"})
            assert r.status_code == 200, r
            assert r.data["user"]["username"] == "pat1"
        for login in (pat2.username, pat2.email):
            r = self.client.post(reverse('api-login'),
                                 data={"username": login, "password": "pat2"})
            assert r.status_code == 200, (r, login)
            assert r.data["user"]["username"] == "pat2"


class ClariwebUserTestCase(EmptyCacheAPITestCase, BasicTestCase):
    def setUp(self):
        self.med1 = self.createClariwebUser("med1", "MED", 10, civilite="Dr")

    def test_accesslog(self):
        self._test_login("med1", first_name="med1", last_name="med1")
        assert len(AccessLog.objects.filter(user=self.med1)) == 1


class TestHeaderAuthMiddleware(EmptyCacheAPITestCase, BasicTestCase):
    def setUp(self):
        self.med1 = self.createClariwebUser("med1", "MED", 10, civilite="Dr")

    def test_sessionmiddleware(self):
        r = self._test_login("med1", first_name="med1", last_name="med1")
        sessionid = r.data["sessionid"]
        assert len(sessionid) == 32, r
        c = APIClient()
        r = c.get(reverse("document-listall"))
        assert r.status_code == status.HTTP_403_FORBIDDEN, r
        r = c.get(reverse("document-listall"), HTTP_X_SESSION_ID=sessionid)
        assert r.status_code == status.HTTP_200_OK, r


class AbstractTestDocument(EmptyCacheAPITestCase, BasicTestCase):
    def setUp(self):
        super().setUp()
        tests_file_path = os.path.join(settings.DOCUMENTS_ROOT, "tests")
        dest_test_filepath = os.path.join(tests_file_path, "test1.pdf")
        if not os.path.isdir(tests_file_path):
            os.mkdir(tests_file_path)
            shutil.copy(os.path.join(settings.BASE_DIR, "tests", "test1.pdf"),
                        dest_test_filepath)
        self.test_fpath = dest_test_filepath
        self.med1 = self.createClariwebUser(
            "med1", "MED", 10, civilite="Dr",
            first_name="laurent", last_name="COUSTET")
        self.med2 = self.createClariwebUser(
            "med2", "MED", 11, civilite="Dr",
            first_name="patrick", last_name="MARTINET")
        self.cor1 = self.createClariwebUser(
            "cor1", "COR", 40, first_name="CLINIQUE", last_name="MONTROUGE",
            service_clinique="URGENCES")
        self.cor2 = self.createClariwebUser(
            "cor2", "COR", 41, first_name="CLINIQUE", last_name="AUTRESERVICE",
            service_clinique="URGENCES", group_clinic="CLIN1")
        self.patu = self.createClariwebUser(
            "pat", "PAT", 20, first_name="jean", last_name="PAUL",
            service_clinique="FAIL")
        self.pat = self.patu.to_actor()
        self.doc1 = self.createDocument(
            self.patu, self.pat, 42, medecin=self.med1.to_actor())
        self.doc2 = self.createDocument(
            self.patu, self.pat, 43, medecin=self.med1.to_actor())
        self.doc3 = self.createDocument(self.patu, self.pat, 44)
        self.doc4 = self.createDocument(
            self.med1, self.pat, 45, medecin=self.med1.to_actor())
        self.doc5 = self.createDocument(
            self.med2, self.pat, 46, medecin=self.med2.to_actor())
        self.doc6 = self.createDocument(
            self.cor1, self.pat, 47, mdecin=self.med2.to_actor())


class LoginUsingInvoiceTestCase(EmptyCacheAPITestCase, BasicTestCase):
    def setUp(self):
        super().setUp()
        tests_file_path = os.path.join(settings.DOCUMENTS_ROOT, "tests")
        dest_test_filepath = os.path.join(tests_file_path, "test1.pdf")
        if not os.path.isdir(tests_file_path):
            os.mkdir(tests_file_path)
            shutil.copy(os.path.join(settings.BASE_DIR, "tests", "test1.pdf"),
                        dest_test_filepath)
        self.test_fpath = dest_test_filepath
        self.patu = self.createClariwebUser(
            "pat", "PAT", 20, first_name="jean", last_name="PAUL",
            date_naissance=datetime.date(1985, 8, 12))
        self.patu2 = self.createClariwebUser(
            "pat2", "PAT", 21, first_name="user2", last_name="PATRICK",
            date_naissance=datetime.date(1999, 9, 9))
        self.pat = self.patu.to_actor()
        self.pat2 = self.patu2.to_actor()
        self.doc1 = self.createDocument(self.patu, self.pat, 42)
        self.doc1.is_invoice = True
        self.doc1.invoice_id = "520244214"
        self.doc1.save()
        self.doc2 = self.createDocument(self.patu, self.pat, 43)
        self.doc2.is_invoice = True
        self.doc2.invoice_id = "500021255"
        self.doc2.save()
        self.doc3 = self.createDocument(self.patu, self.pat, 44)
        self.doc3.is_invoice = False
        self.doc3.save()
        self.doc4 = self.createDocument(self.patu2, self.pat2, 45)
        self.doc4.invoice_id = "500021256"
        self.doc4.save()

    def _login(self, invoice_id, birthdate, shouldpass=True):
        response = self.client.post(
            reverse("api-login-invoice"),
            {"invoice_id": invoice_id, "birthdate": birthdate})
        if shouldpass:
            assert response.status_code == status.HTTP_200_OK
        if response.status_code == status.HTTP_200_OK:
            self.client.credentials(
                HTTP_X_SESSION_ID=response.data["sessionid"])
        return response

    def test_login_bad_data(self):
        r = self.client.post(
            reverse("api-login-invoice"),
            {"invoice_id": "123"})
        assert r.status_code == status.HTTP_400_BAD_REQUEST
        assert "errors" in r.data

    def test_login_fail_bad_invoice(self):
        r = self._login("12123", datetime.date(1999, 1, 1), shouldpass=False)
        assert r.status_code == status.HTTP_401_UNAUTHORIZED

    def test_login_fail_bad_secu(self):
        r = self._login("520244214",
                        datetime.date(1999, 9, 9), shouldpass=False)
        assert r.status_code == status.HTTP_401_UNAUTHORIZED

    def test_login_fail_allbad(self):
        r = self._login("54214", datetime.date(2001, 1, 1), shouldpass=False)
        assert r.status_code == status.HTTP_401_UNAUTHORIZED

    def test_login_good(self):
        for invoice_id in ("520244214", "500021255"):
            r = self._login(invoice_id, datetime.date(1985, 8, 12))
            assert r.data["user"]["invoice_only"]

    def test_document_list(self):
        self._login("520244214", datetime.date(1985, 8, 12))
        # Acces invoice 1 OK
        r = self.client.get(reverse("document-download",
                            kwargs={'document_pk': self.doc1.pk}))
        assert r.status_code == status.HTTP_200_OK
        # Access invoice 2 OK
        r = self.client.get(reverse("document-download",
                            kwargs={'document_pk': self.doc2.pk}))
        assert r.status_code == status.HTTP_200_OK
        # Access CR 3 NOK
        r = self.client.get(reverse("document-download",
                            kwargs={'document_pk': self.doc3.pk}))
        assert r.status_code == status.HTTP_404_NOT_FOUND
        # Listall should see doc 1 and 2
        r = self.client.get(reverse("document-listall"))
        assert r.status_code == status.HTTP_200_OK
        assert len(r.data["results"]) == 2
        assert [d["id"] for d in r.data["results"]] == [
            self.doc2.pk, self.doc1.pk]


class TestDocumentList(AbstractTestDocument):
    def test_document_download_accesslog(self):
        self._test_login("pat", first_name="jean", last_name="PAUL")
        access_logs = DocumentAccessLog.objects.filter(
            user=self.patu.clariwebuser)
        assert len(access_logs) == 0
        url = reverse("document-download",
                      kwargs={'document_pk': self.doc1.pk})
        r = self.client.get(url)
        r = self.client.get(url)  # Do it twice to log twice
        assert r.status_code == status.HTTP_200_OK
        access_logs = DocumentAccessLog.objects.filter(
            user=self.patu.clariwebuser)
        assert len(access_logs) == 2
        assert access_logs[0].document.sil_demande_pk == 42

    def test_document_listall(self):
        self._test_login("pat", first_name="jean", last_name="PAUL")
        # Generate access log
        r = self.client.get(reverse("document-download",
                            kwargs={'document_pk': self.doc1.pk}))
        assert r.status_code == status.HTTP_200_OK
        # Request listall
        r = self.client.get(reverse("document-listall"))
        assert r.status_code == status.HTTP_200_OK
        # Should be all documents for this user.
        assert len(r.data["results"]) == 3

    def test_document_search_patient(self):
        self._test_login("pat", first_name="jean", last_name="PAUL")
        # Request listall
        r = self.client.get(reverse("document-listall"), {
            "search": "JEAN pAuL"
        })
        assert r.status_code == status.HTTP_200_OK
        assert len(r.data["results"]) == 3

    def test_document_patient_no_service_clinic(self):
        self._test_login("pat", first_name="jean", last_name="PAUL")
        r = self.client.get(reverse("document-listall"), {
            "search": "paul jean"
        })
        assert r.status_code == status.HTTP_200_OK, r.data
        assert len(r.data["results"]) == 3
        assert not r.data["results"][0]["user"]["service_clinic"]

    def test_document_search_patient_unaccent(self):
        self._test_login("pat", first_name="jean", last_name="PAUL")
        # Request listall
        r = self.client.get(reverse("document-listall"), {
            "search": "JéàN pAuL"
        })
        assert r.status_code == status.HTTP_200_OK
        assert len(r.data["results"]) == 3

    def test_document_correspondant(self):
        self._test_login("cor1", first_name="CLINIQUE", last_name="MONTROUGE")
        # Request listall
        r = self.client.get(reverse("document-listall"))
        assert r.status_code == status.HTTP_200_OK
        assert len(r.data["results"]) == 1
        doc = r.data["results"][0]
        assert self.cor1.clariwebuser.service_clinic
        assert doc["user"]["service_clinic"] == self.cor1.clariwebuser.service_clinic  # noqa

    def test_document_correspondant_clinique_badip(self):
        constance.config.SECURITY_IP_ADDRESSES_CLINIC = ["10.0.0.0/8"]
        r = self._test_login(
            "cor2", first_name="CLINIQUE", last_name="AUTRESERVICE",
            shouldpass=False)
        assert r.status_code == status.HTTP_401_UNAUTHORIZED

    def test_document_correspondant_clinique_ok(self):
        constance.config.SECURITY_IP_ADDRESSES_CLINIC = ["127.0.0.0/8"]
        constance.config.CLINIC_SHARE_SERVICES = True
        self.cor1.clariwebuser.group_clinic = self.cor2.clariwebuser.group_clinic  # noqa
        self.cor1.clariwebuser.save()
        r = self._test_login(
            "cor2", first_name="CLINIQUE", last_name="AUTRESERVICE",
            REMOTE_ADDR="127.0.0.1")
        r = self.client.get(reverse("document-listall"))
        assert r.status_code == status.HTTP_200_OK
        assert len(r.data["results"]) == 1

    def test_document_correspondant_clinique_noshare(self):
        constance.config.SECURITY_IP_ADDRESSES_CLINIC = ["127.0.0.0/8"]
        constance.config.CLINIC_SHARE_SERVICES = False
        self.cor1.clariwebuser.group_clinic = self.cor2.clariwebuser.group_clinic  # noqa
        self.cor1.clariwebuser.save()
        r = self._test_login(
            "cor2", first_name="CLINIQUE", last_name="AUTRESERVICE",
            REMOTE_ADDR="127.0.0.1")
        r = self.client.get(reverse("document-listall"))
        assert r.status_code == status.HTTP_200_OK
        assert len(r.data["results"]) == 0

    def test_document_listnotseen(self):
        self._test_login("pat", first_name="jean", last_name="PAUL")
        # Generate access log
        r = self.client.get(reverse("document-download",
                                    kwargs={'document_pk': self.doc2.pk}))
        assert r.status_code == status.HTTP_200_OK
        r = self.client.get(reverse("document-listnotseen"))
        assert r.status_code == status.HTTP_200_OK
        # Should be all documents for this user
        # minus the one we just downloaded
        assert len(r.data["results"]) == 2
        # List should be ordered "foreign document (group)" first
        assert [d["id"] for d in r.data["results"]] == [
            self.doc3.pk, self.doc1.pk]
        # Read the rest
        r = self.client.get(reverse("document-download",
                            kwargs={'document_pk': self.doc1.pk}))
        r = self.client.get(reverse("document-download",
                            kwargs={'document_pk': self.doc3.pk}))
        r = self.client.get(reverse("document-listnotseen"))
        assert r.status_code == status.HTTP_200_OK
        # Nothing is expected.
        assert len(r.data["results"]) == 0

    def test_document_listnotseen_multiuser(self):
        self._test_login("pat", first_name="jean", last_name="PAUL")
        # Generate access log
        r = self.client.get(reverse("document-download",
                            kwargs={'document_pk': self.doc2.pk}))
        assert r.status_code == status.HTTP_200_OK
        # Relogin as medecin1
        self._test_login("med1", first_name="laurent", last_name="COUSTET")
        r = self.client.get(reverse("document-listnotseen"))
        assert r.status_code == status.HTTP_200_OK
        assert len(r.data["results"]) == 1
        assert [d["id"] for d in r.data["results"]] == [self.doc4.pk, ]
        # Read one
        r = self.client.get(reverse("document-download",
                            kwargs={'document_pk': self.doc1.pk}))
        r = self.client.get(reverse("document-listnotseen"))
        assert r.status_code == status.HTTP_200_OK
        # Nothing is expected.
        assert len(r.data["results"]) == 1

    def test_document_list_foreignorder(self):
        self.med3 = self.createClariwebUser("med3", "MED", 30, group="MEDS")
        self.med4 = self.createClariwebUser("med4", "MED", 31, group="MEDS")
        docs = [
            self.createDocument(self.med3, self.pat, 55,
                                medecin=self.med3.to_actor()),
            self.createDocument(self.med4, self.pat, 56,
                                medecin=self.med4.to_actor())
        ]

        self._test_login("med4")
        r = self.client.get(reverse("document-listall"))
        assert r.status_code == status.HTTP_200_OK
        assert len(r.data["results"]) == 2
        # Order should be foreign last
        assert [d["id"] for d in r.data["results"]] == [docs[1].pk, docs[0].pk]

    def test_document_download_permission(self):
        self._test_login("med1", first_name="laurent", last_name="COUSTET")
        r = self.client.get(reverse("document-download",
                            kwargs={'document_pk': self.doc1.pk}))
        assert r.status_code == status.HTTP_404_NOT_FOUND, r

    def test_document_download(self):
        self._test_login("med1", first_name="laurent", last_name="COUSTET")
        r = self.client.get(reverse("document-download",
                            kwargs={'document_pk': self.doc4.pk}))
        assert r.status_code == status.HTTP_200_OK

    def test_document_download_unpaid_med(self):
        self.doc4.unpaid_amount = 3000
        self.doc4.save()
        self._test_login("med1", first_name="laurent", last_name="COUSTET")
        r = self.client.get(reverse("document-download",
                            kwargs={'document_pk': self.doc4.pk}))
        assert r.status_code == status.HTTP_200_OK

    def test_document_download_unpaid_pat(self):
        constance.config.DEBPT_MINIMUM_AMOUNT = 100
        self.patu.clariwebuser.debpt = 3000
        self.patu.clariwebuser.save()
        self.doc1.unpaid_amount = 3000
        self.doc1.save()
        self._test_login("pat", first_name="jean", last_name="PAUL")
        r = self.client.get(reverse("document-download",
                            kwargs={'document_pk': self.doc1.pk}))
        assert r.status_code == status.HTTP_404_NOT_FOUND

    def test_document_detail(self):
        self._test_login("pat", first_name="jean", last_name="PAUL")
        r = self.client.get(reverse("document-detail",
                            kwargs={'pk': self.doc1.pk}))
        assert r.status_code == status.HTTP_200_OK

    def test_document_analyses_list(self):
        self.doc4.analyses = [{
            "code": "TP",
            "anormaux": [],
            "a_saisir": False,
            "libelle": "Taux de protrombine"
        }]
        self.doc4.save()
        d = Document.objects.get(id=self.doc1.pk)
        self._test_login("med1", first_name="laurent", last_name="COUSTET")
        r = self.client.get(reverse("document-listall"))
        assert r.status_code == status.HTTP_200_OK
        for d in r.data["results"]:
            if d["id"] == self.doc4.pk:
                assert self.doc4.analyses == d["analyses"]
                return
        assert not True

    def test_document_analyses_list_not_for_patient(self):
        self.doc1.analyses = [{
            "code": "TP",
            "anormaux": [],
            "a_saisir": False,
            "libelle": "Taux de protrombine"
        }]
        self.doc1.save()
        d = Document.objects.get(id=self.doc1.pk)
        self._test_login("pat", first_name="jean", last_name="PAUL")
        r = self.client.get(reverse("document-listall"))
        assert r.status_code == status.HTTP_200_OK
        for d in r.data["results"]:
            assert len(d["analyses"]) == 0

    def test_document_analyses_detail(self):
        self._test_login("pat", first_name="jean", last_name="PAUL")
        r = self.client.get(reverse("document-detail",
                            kwargs={'pk': self.doc1.pk}))
        assert r.status_code == status.HTTP_200_OK
        assert r.data["analyses"] == []


class TestDocumentClinicList(AbstractTestDocument):
    def test_shareservice(self):
        constance.config.SECURITY_IP_ADDRESSES_CLINIC = ["127.0.0.0/8"]
        constance.config.CLINIC_SHARE_SERVICES = True
        self.cor1.clariwebuser.group_clinic = self.cor2.clariwebuser.group_clinic  # noqa
        self.cor1.clariwebuser.save()
        r = self._test_login(
            "cor2", first_name="CLINIQUE", last_name="AUTRESERVICE",
            REMOTE_ADDR="127.0.0.1")
        r = self.client.get(reverse("document-clinic-list"))
        assert r.status_code == status.HTTP_200_OK
        assert len(r.data["results"]) == 1

    def test_notcompterendu(self):
        self.doc6.document = "Autre Document"
        self.doc6.save()
        r = self._test_login(
            "cor1", first_name="CLINIQUE", last_name="MONTROUGE",
            REMOTE_ADDR="127.0.0.1")
        r = self.client.get(reverse("document-clinic-list"))
        assert r.status_code == status.HTTP_200_OK
        assert len(r.data["results"]) == 0

    def test_apinoshare(self):
        constance.config.SECURITY_IP_ADDRESSES_CLINIC = ["127.0.0.0/8"]
        constance.config.CLINIC_SHARE_SERVICES = True
        self.cor1.clariwebuser.group_clinic = self.cor2.clariwebuser.group_clinic  # noqa
        self.cor1.clariwebuser.save()
        r = self._test_login(
            "cor2", first_name="CLINIQUE", last_name="AUTRESERVICE",
            REMOTE_ADDR="127.0.0.1")
        r = self.client.get(reverse("document-clinic-list-noshareservice"))
        assert r.status_code == status.HTTP_200_OK
        assert len(r.data["results"]) == 0


class TestDocumentListDebpt(AbstractTestDocument):
    def setUp(self):
        super().setUp()
        self.patu.clariwebuser.debpt = 500
        self.patu.clariwebuser.save()
        self.doc3.is_invoice = True
        self.doc3.save()

    def test_debpt_disabled(self):
        constance.config.DEBPT_MINIMUM_AMOUNT = 0
        self._test_login("pat", first_name="jean", last_name="PAUL")
        r = self.client.get(reverse("document-listall"))
        assert r.status_code == status.HTTP_200_OK
        assert len(r.data["results"]) == 3

    def test_debpt_below_minimum(self):
        constance.config.DEBPT_MINIMUM_AMOUNT = 1000
        self._test_login("pat", first_name="jean", last_name="PAUL")
        r = self.client.get(reverse("document-listall"))
        assert r.status_code == status.HTTP_200_OK
        assert len(r.data["results"]) == 3

    def test_debpt_above_minimum(self):
        constance.config.DEBPT_MINIMUM_AMOUNT = 100
        self._test_login("pat", first_name="jean", last_name="PAUL")
        r = self.client.get(reverse("document-listall"))
        assert r.status_code == status.HTTP_200_OK
        assert len(r.data["results"]) == 1
        # Order should be foreign last
        assert [d["id"] for d in r.data["results"]] == [self.doc3.pk]


class EmailConfirmTestCase(EmptyCacheAPITestCase, BasicTestCase):
    def test_empty_key(self):
        url = reverse("api-emailconfirm")
        for key in (None, "", "1234", "az-`'%"):
            r = self.client.post(url, {"key": key})
            assert r.status_code == status.HTTP_400_BAD_REQUEST
            assert r.data == {"error": "Clef incorrecte"}

    def test_key_not_found(self):
        url = reverse("api-emailconfirm")
        key = "0123456789ab"
        r = self.client.post(url, {"key": key})
        assert r.status_code == status.HTTP_400_BAD_REQUEST
        assert r.data == {"error": "Clef incorrecte"}

    def test_key_expired(self):
        user = self.createClariwebUser("test", "PAT", 30,
                                       civilite="Mr",
                                       first_name="Laurent",
                                       last_name="Coustet",
                                       email="test@test.com")
        user.add_unconfirmed_email(user.email)
        address = EmailAddress.objects.get(user=user, email=user.email)
        address.set_at = address.set_at - datetime.timedelta(
            days=settings.SIMPLE_EMAIL_CONFIRMATION_PERIOD.days + 1)
        address.save()
        url = reverse("api-emailconfirm")
        key = user.confirmation_key
        r = self.client.post(url, {"key": key})
        assert r.status_code == status.HTTP_404_NOT_FOUND, r
        assert r.data == {"expired": True}

    def test_valid_key(self):
        user = self.createClariwebUser("test", "PAT", 30,
                                       civilite="Mr",
                                       first_name="Laurent",
                                       last_name="Coustet",
                                       email="test@test.com")
        url = reverse("api-emailconfirm")
        user.add_unconfirmed_email(user.email)
        key = user.confirmation_key
        r = self.client.post(url, data={"key": key})
        assert r.status_code == status.HTTP_200_OK, r
        assert r.data == {
            "user": {
                "email": user.email,
                "full_name": user.get_full_name(),
            }
        }

    def test_resend_email_notloggedin(self):
        self.createClariwebUser("test", "PAT", 30,
                                civilite="Mr",
                                first_name="Laurent",
                                last_name="Coustet",
                                email="test@test.com")
        url = reverse("api-resendemailconfirm")
        r = self.client.get(url)
        assert r.status_code == status.HTTP_403_FORBIDDEN

    def test_resend_email_validuser(self):
        user = self.createClariwebUser(
            "test", "PAT", 30,
            civilite="Mr",
            email="test@test.com")
        user.add_unconfirmed_email(user.email)
        self._test_login("test", email='test@test.com')
        mail.outbox = []
        url = reverse("api-resendemailconfirm")
        r = self.client.get(url)
        assert r.status_code == status.HTTP_200_OK, r
        assert len(mail.outbox) == 1

    def test_resend_email_expired(self):
        user = self.createClariwebUser("test", "PAT", 30,
                                       civilite="Mr",
                                       email="test@test.com")
        user.add_unconfirmed_email(user.email)
        self._test_login("test", email='test@test.com')
        address = EmailAddress.objects.get(user=user, email=user.email)
        address.set_at = address.set_at - datetime.timedelta(
            days=settings.SIMPLE_EMAIL_CONFIRMATION_PERIOD.days + 1)
        address.save()
        assert address.is_key_expired
        mail.outbox = []
        url = reverse("api-resendemailconfirm")
        r = self.client.get(url)
        assert r.status_code == status.HTTP_200_OK, r
        address = EmailAddress.objects.get(user=user, email=user.email)
        assert not address.is_key_expired


class RateLimitTestCase(EmptyCacheAPITestCase, BasicTestCase):
    def setUp(self):
        super().setUp()
        SMSCounter.set_quota_left(100)

    def test_login_ratelimit(self):
        self.createClariwebUser("test", "PAT", 30,
                                civilite="Mr",
                                email="test@test.com")
        data = {"username": "test", "password": "test"}
        url = reverse("api-login")
        for i in range(5):
            response = self.client.post(url, data)
            assert response.status_code == status.HTTP_200_OK
        response = self.client.post(url, data)
        assert response.status_code == status.HTTP_429_TOO_MANY_REQUESTS

    def test_login_invoice_ratelimit(self):
        self.createClariwebUser("test", "PAT", 30,
                                civilite="Mr",
                                email="test@test.com",
                                date_naissance=datetime.date(1999, 9, 9))
        data = {
            "invoice_id": "2412414",
            "birthdate": datetime.date(1999, 9, 9)
        }
        url = reverse("api-login-invoice")
        for i in range(5):
            response = self.client.post(url, data)
            assert response.status_code == status.HTTP_401_UNAUTHORIZED
        response = self.client.post(url, data)
        assert response.status_code == status.HTTP_429_TOO_MANY_REQUESTS

    def test_emailresend_ratelimit(self):
        user = self.createClariwebUser(
            "test", "PAT", 30,
            civilite="Mr",
            email="test@test.com")
        user.add_unconfirmed_email(user.email)
        self._test_login("test", email='test@test.com')
        url = reverse("api-resendemailconfirm")
        for i in range(5):
            response = self.client.get(url)
            assert response.status_code == status.HTTP_200_OK
        response = self.client.get(url)
        assert response.status_code == status.HTTP_429_TOO_MANY_REQUESTS

    def test_passwordreset_ask(self):
        data = {
            "first_name": "Bonjour",
            "last_name": "Monde",
            "birthdate": datetime.date(1980, 4, 1)
        }
        for i in range(20):
            response = self.client.post(
                reverse("api-passwordreset-ask"), data)
            assert response.status_code == status.HTTP_404_NOT_FOUND
        response = self.client.post(reverse("api-passwordreset-ask"), data)
        assert response.status_code == status.HTTP_429_TOO_MANY_REQUESTS


class AdminViewsTestCase(AdminTestCase, BasicTestCase):
    def setUp(self):
        super().setUp()
        self.med1 = self.createClariwebUser("med1", "MED", 10, civilite="Dr")
        self.med2 = self.createClariwebUser("med2", "MED", 11, civilite="Dr")
        self.notificationsdict = [
            {
                "status": "ERROR",
                "error": "Invalid number",
                "user": self.med1.clariwebuser,
                "date": timezone.now() - datetime.timedelta(days=1),
                "sms": "+33611111155"
            },
            {
                "status": "SENT",
                "user": self.med1.clariwebuser,
                "date": timezone.now(),
                "sms": "+33611111155"
            },
            {
                "status": "SENT",
                "user": self.med2.clariwebuser,
                "date": timezone.now(),
                "sms": "+33611111156"
            },
            {
                "status": "SENT",
                "user": self.med2.clariwebuser,
                "date": timezone.now(),
                "email": "med@med1.fr"
            }
        ]
        for notification in self.notificationsdict:
            Notification.objects.create(**notification)

    def test_views_notadmin(self):
        user = self.createClariwebUser("test", "PAT", 30,
                                       civilite="Mr",
                                       email="test@test.com")
        assert not user.is_staff
        self._test_login("test", email='test@test.com')
        for url in ("api-admin-notification",
                    "api-admin-accesslog",
                    "api-admin-smscount",
                    "api-admin-smscount-getquota",
                    "api-admin-smscount-updatequota",
                    "api-admin-settings",
                    "api-admin-stats-document-month-user",
                    "api-admin-stats-document-month-graph",
                    "api-admin-payments"):
            response = self.client.get(reverse(url))
            assert response.status_code == status.HTTP_403_FORBIDDEN
        for url in ("api-admin-smscount-buy", ):
            response = self.client.post(reverse(url))
            assert response.status_code == status.HTTP_403_FORBIDDEN

    def _common_filtering(self):
        user = self.createClariwebUser("test", "PAT", 30,
                                       civilite="Mr",
                                       email="test@test.com")
        user.is_staff = True
        user.save()
        assert user.is_staff
        self._test_login("test", email='test@test.com')

    def test_notification_filtering_all(self):
        self._common_filtering()
        response = self.client.get(reverse("api-admin-notification"))
        assert response.status_code == status.HTTP_200_OK
        assert len(response.data["results"]) == 4

    def test_notification_filtering_status(self):
        self._common_filtering()
        response = self.client.get(
            reverse("api-admin-notification"),
            {"search": "ERROR"})
        assert response.status_code == status.HTTP_200_OK
        assert len(response.data["results"]) == 1

    def test_notification_filtering_user(self):
        self._common_filtering()
        response = self.client.get(
            reverse("api-admin-notification"),
            {"search": "med42"})
        assert response.status_code == status.HTTP_200_OK
        assert len(response.data["results"]) == 0
        response = self.client.get(
            reverse("api-admin-notification"),
            {"search": "med1"})
        assert response.status_code == status.HTTP_200_OK
        assert len(response.data["results"]) == 2

    def test_notification_filtering_combined(self):
        self._common_filtering()
        response = self.client.get(
            reverse("api-admin-notification"),
            {"search": "med1 ERROR"})
        assert response.status_code == status.HTTP_200_OK
        assert len(response.data["results"]) == 1

    def test_notification_ordering(self):
        self._common_filtering()
        response = self.client.get(
            reverse("api-admin-notification"),
            {"search": "med1", "ordering": "-date"})
        assert response.status_code == status.HTTP_200_OK
        assert len(response.data["results"]) == 2
        results = response.data["results"]
        assert results[0]["date"] > results[1]["date"]
        response = self.client.get(
            reverse("api-admin-notification"),
            {"search": "med1", "ordering": "date"})
        assert response.status_code == status.HTTP_200_OK
        assert len(response.data["results"]) == 2
        results = response.data["results"]
        assert results[1]["date"] > results[0]["date"]

    def test_notification_pagination(self):
        for i in range(210 - len(self.notificationsdict)):
            Notification.objects.create(
                **self.notificationsdict[i % len(self.notificationsdict)])
        self._common_filtering()
        response = self.client.get(reverse("api-admin-notification"))
        assert response.status_code == status.HTTP_200_OK
        results = response.data["results"]
        assert len(results) == 50
        response = self.client.get(
            reverse("api-admin-notification"),
            {"page": 5})
        results = response.data["results"]
        assert len(results) == 10

    def test_payments_simple(self):
        self._common_filtering()
        response = self.client.get(reverse("api-admin-payments"))
        assert response.status_code == status.HTTP_200_OK
        results = response.data["results"]
        assert len(results) == 0

    def test_smscount(self):
        self._common_filtering()
        response = self.client.get(reverse('api-admin-smscount'))
        assert response.data["smscount_total"] == 2

    def test_sms_get_quota(self):
        self._common_filtering()
        response = self.client.get(reverse('api-admin-smscount-getquota'))
        assert response.status_code == status.HTTP_200_OK
        assert isinstance(response.data["sms_quota"], six.integer_types)

    # Disabled because it crashes the jenkins test
    # def test_sms_update_quota(self):
    #     self._common_filtering()
    #     response = self.client.get(reverse('api-admin-smscount-updatequota'))
    #     self.assertEqual(response.status_code, status.HTTP_200_OK)
    #     self.assertTrue(isinstance(response.data["sms_quota"], (int, long)))

    def test_sms_buy_badquantity(self):
        self._common_filtering()
        response = self.client.post(reverse('api-admin-smscount-buy'))
        assert response.status_code == status.HTTP_400_BAD_REQUEST
        response = self.client.post(reverse('api-admin-smscount-buy'),
                                    {'quantity': 1})
        assert response.status_code == status.HTTP_400_BAD_REQUEST

    def test_sms_buy(self):
        self._common_filtering()
        mail.outbox = []
        for quantity in (5000, 10000, 20000):
            response = self.client.post(reverse('api-admin-smscount-buy'),
                                        {'quantity': quantity})
            assert response.status_code == status.HTTP_200_OK, response.data
        assert len(mail.outbox) == 3, [m.subject for m in mail.outbox]


class AdminLoginAsTestCase(AdminTestCase, BasicTestCase):
    def setUp(self):
        self.staffuser = self.createClariwebUser("staff", "MED", 10)
        self.staffuser.is_staff = True
        self.staffuser.is_superuser = False
        self.staffuser.save()
        self.staff = self.staffuser.to_actor()
        self.staff.save()
        self.notstaffuser = self.createClariwebUser("notstaff", "PAT", 30)
        self.notstaffuser.is_staff = False
        self.notstaffuser.is_superuser = False
        self.notstaff = self.notstaffuser.to_actor()
        self.notstaff.save()
        tests_file_path = os.path.join(settings.DOCUMENTS_ROOT, "tests")
        dest_test_filepath = os.path.join(tests_file_path, "test1.pdf")
        if not os.path.isdir(tests_file_path):
            os.mkdir(tests_file_path)
            shutil.copy(os.path.join(settings.BASE_DIR, "tests", "test1.pdf"),
                        dest_test_filepath)
        self.test_fpath = dest_test_filepath
        self.doc1 = self.createDocument(self.notstaffuser, self.notstaff, 42)
        self.doc2 = self.createDocument(self.staffuser, self.staff, 43)
        super().setUp()

    def test_views_notadmin(self):
        assert not self.notstaffuser.is_staff
        self._test_login("notstaff")
        for url in ("api-admin-loginas-selector", ):
            response = self.client.get(reverse(url))
            assert response.status_code == status.HTTP_403_FORBIDDEN
        for url in ("api-admin-loginas", ):
            response = self.client.post(reverse(url))
            assert response.status_code == status.HTTP_403_FORBIDDEN

    def test_selector_basic(self):
        self._test_login("staff")
        response = self.client.get(reverse("api-admin-loginas-selector"))
        assert response.status_code == status.HTTP_200_OK
        assert len(response.data["results"]) == 2

    def test_selector_query_first_name(self):
        self.createClariwebUser("pat1", "PAT", 31, first_name="Laurent",
                                last_name="COUSTET")
        self._test_login("staff")
        response = self.client.get(
            reverse("api-admin-loginas-selector"),
            {"search": "lau"}
        )
        assert response.status_code == status.HTTP_200_OK
        assert len(response.data["results"]) == 1
        u = response.data["results"][0]
        assert u["user"]["username"] == "pat1"

    def test_selector_query_last_name(self):
        self.createClariwebUser("pat1", "PAT", 31, first_name="Laurent",
                                last_name="COUSTET")
        self._test_login("staff")
        response = self.client.get(
            reverse("api-admin-loginas-selector"),
            {"search": "cOust"}
        )
        assert response.status_code == status.HTTP_200_OK
        assert len(response.data["results"]) == 1
        u = response.data["results"][0]
        assert u["user"]["username"] == "pat1"

    def test_selector_query_combined(self):
        self.createClariwebUser("pat1", "PAT", 31, first_name="Laurent",
                                last_name="COUSTET")
        self._test_login("staff")
        response = self.client.get(
            reverse("api-admin-loginas-selector"),
            {"search": "coust Pat"}
        )
        assert response.status_code == status.HTTP_200_OK
        assert len(response.data["results"]) == 1
        u = response.data["results"][0]
        assert u["user"]["username"] == "pat1"

    def test_loginas_badrequest(self):
        self._test_login("staff")
        response = self.client.post(reverse("api-admin-loginas"))
        assert response.status_code == status.HTTP_400_BAD_REQUEST

    def test_loginas_staff_on_superuser(self):
        su = self.createClariwebUser("su", "MED", 40)
        su.is_staff = True
        su.is_superuser = True
        su.save()
        assert su.is_superuser
        assert not self.staffuser.is_superuser
        self._test_login("staff")
        response = self.client.post(reverse("api-admin-loginas"),
                                    {"username": "su"})
        assert response.status_code == status.HTTP_403_FORBIDDEN

    def test_loginas_baduser(self):
        self._test_login("staff")
        response = self.client.post(reverse("api-admin-loginas"),
                                    {"username": "inexistant"})
        assert response.status_code == status.HTTP_404_NOT_FOUND

    def test_loginas_good(self):
        self.staffuser.is_superuser = True
        self.staffuser.save()
        self._test_login("staff")
        response = self.client.post(reverse("api-admin-loginas"),
                                    {"username": "notstaff"})
        assert response.status_code == status.HTTP_200_OK
        assert response.data["username"] == "notstaff"
        response = self.client.get(reverse('document-listall'))
        assert response.status_code == status.HTTP_200_OK
        assert len(response.data["results"]) == 1
        # Rollback
        response = self.client.post(reverse("api-admin-loginas"),
                                    {"username": "staff"})
        assert response.status_code == status.HTTP_200_OK
        assert response.data["username"] == "staff"
        response = self.client.get(reverse('document-listall'))
        assert response.status_code == status.HTTP_200_OK
        assert len(response.data["results"]) == 2


class AdminSettingsTestCase(AdminTestCase, BasicTestCase):
    def setUp(self):
        super().setUp()
        self.staffuser = self.createClariwebUser("staff", "MED", 10)
        self.staffuser.is_staff = True
        self.staffuser.save()

    def test_list_settings(self):
        self._test_login("staff")
        response = self.client.get(reverse("api-admin-settings"))
        assert response.status_code == status.HTTP_200_OK
        assert len(response.data) == len(constance.settings.CONFIG.keys())
        key1 = response.data[0]
        assert key1["help_text"] == constance.settings.CONFIG[key1["key"]][1]

    def test_modify_settings_badkey(self):
        self._test_login("staff")
        response = self.client.post(reverse("api-admin-settings"))
        assert response.status_code == status.HTTP_400_BAD_REQUEST
        response = self.client.post(reverse("api-admin-settings"),
                                    {"key": "TARATATA"})
        assert response.status_code == status.HTTP_400_BAD_REQUEST

    def test_modify_settings(self):
        self._test_login("staff")
        items = (
            ("CUSTOM_ADDRESS_LINE1", "La rue enchantée"),
        )
        response = self.client.post(
            reverse("api-admin-settings"),
            {"items": [{"key": k, "value": v} for k, v in items]})
        assert response.status_code == status.HTTP_200_OK


class PublicSettingsTestCase(EmptyCacheAPITestCase, BasicTestCase):
    def test_api_settings(self):
        keys = ("DEBPT_MINIMUM_AMOUNT", "CLINIC_ENABLED")
        response = self.client.get(reverse("api-settings"))
        assert response.status_code == status.HTTP_200_OK
        assert all([
            k.startswith("CUSTOM") or k in keys for k in response.data.keys()])


class PasswordResetTestCase(EmptyCacheAPITestCase, BasicTestCase):
    def setUp(self):
        self.u1 = self.createClariwebUser(
            "user1", "PAT", 1,
            date_naissance=datetime.date(1980, 4, 1),
            address_mobile_phone="+336000000000")
        self.u2 = self.createClariwebUser(
            "user2", "PAT", 2, password="user2",
            date_naissance=datetime.date(2001, 1, 1))
        self.u3 = self.createClariwebUser(
            "user3", "PAT", 3, first_name="Émmanuel", last_name="àlarùe",
            date_naissance=datetime.date(2001, 1, 1),
            address_mobile_phone="+336000000000")
        SMSCounter.set_quota_left(100)

    def test_full_pass(self):
        url = reverse("api-passwordreset-ask")
        response = self.client.post(url, {
            "first_name": self.u1.first_name,
            "last_name": self.u1.last_name,
            "birthdate": datetime.date(1980, 4, 1)
        })
        assert response.status_code == status.HTTP_200_OK
        # Then we should have got the validation code from another media.
        pr = PasswordReset.objects.get(user__pk=self.u1.pk)
        url = reverse("api-passwordreset-check")
        response = self.client.post(url, {
            "username": self.u1.username,
            "validation_code": pr.validation_code
        })
        assert response.status_code == status.HTTP_200_OK
        url = reverse("api-passwordreset-edit")
        response = self.client.post(url, {
            "username": self.u1.username,
            "validation_code": pr.validation_code,
            "password": "1234vgdsd!"
        })
        assert response.status_code == status.HTTP_200_OK
        self._test_login("user1", password="1234vgdsd!")

    def test_too_short(self):
        url = reverse("api-passwordreset-ask")
        response = self.client.post(url, {
            "first_name": self.u1.first_name,
            "last_name": self.u1.last_name,
            "birthdate": datetime.date(1980, 4, 1)
        })
        assert response.status_code == status.HTTP_200_OK
        # Then we should have got the validation code from another media.
        pr = PasswordReset.objects.get(user__pk=self.u1.pk)
        url = reverse("api-passwordreset-check")
        response = self.client.post(url, {
            "username": self.u1.username,
            "validation_code": pr.validation_code
        })
        assert response.status_code == status.HTTP_200_OK
        url = reverse("api-passwordreset-edit")
        response = self.client.post(url, {
            "username": self.u1.username,
            "validation_code": pr.validation_code,
            "password": "1"
        })
        assert response.status_code == status.HTTP_400_BAD_REQUEST

    def test_edit_invalid_serializer(self):
        url = reverse("api-passwordreset-ask")
        response = self.client.post(url, {
            "first_name": self.u1.first_name,
            "last_name": self.u1.last_name,
            "birthdate": datetime.date(1980, 4, 1)
        })
        assert response.status_code == status.HTTP_200_OK
        # Then we should have got the validation code from another media.
        pr = PasswordReset.objects.get(user__pk=self.u1.pk)
        url = reverse("api-passwordreset-check")
        response = self.client.post(url, {
            "username": self.u1.username,
            "validation_code": pr.validation_code
        })
        assert response.status_code == status.HTTP_200_OK
        url = reverse("api-passwordreset-edit")
        response = self.client.post(url, {})
        assert response.status_code == status.HTTP_400_BAD_REQUEST

    def test_ask_invalid_serializer(self):
        url = reverse("api-passwordreset-ask")
        response = self.client.post(url, {
            "birthdate": datetime.date(1980, 4, 1)
        })
        assert response.status_code == status.HTTP_400_BAD_REQUEST
        assert "errors" in response.data

    def test_ask_invaliduser(self):
        url = reverse("api-passwordreset-ask")
        for (name, birthdate) in (
                ("alabama", datetime.date(2000, 1, 1)),
                ("isnothere", datetime.date(2004, 1, 1))):
            response = self.client.post(url, {
                "first_name": name,
                "last_name": name,
                "birthdate": birthdate})
            assert response.status_code == status.HTTP_404_NOT_FOUND

    def test_ask_accented(self):
        url = reverse("api-passwordreset-ask")
        response = self.client.post(url, {
            "first_name": "eMmanuel",
            "last_name": "alarue",
            "birthdate": datetime.date(2001, 1, 1)
        })
        assert response.status_code == status.HTTP_200_OK

    def test_ask_bad_birthdate(self):
        url = reverse("api-passwordreset-ask")
        response = self.client.post(url, {
            "first_name": self.u1.first_name,
            "last_name": self.u1.last_name,
            "birthdate": datetime.date(2000, 4, 2)
        })
        assert response.status_code == status.HTTP_404_NOT_FOUND

    def test_ask_smsquotaempty(self):
        SMSCounter.set_quota_left(0)
        url = reverse("api-passwordreset-ask")
        response = self.client.post(url, {
            "first_name": "eMmanuel",
            "last_name": "alarue",
            "birthdate": datetime.date(2001, 1, 1)
        })
        assert response.status_code == status.HTTP_503_SERVICE_UNAVAILABLE

    def test_check_badrequest(self):
        url = reverse("api-passwordreset-check")
        for d in ({"validation_code": 123}, {"username": "pouet"}, {}):
            response = self.client.post(url, d)
            assert response.status_code == status.HTTP_400_BAD_REQUEST

    def test_check_invalidcode(self):
        for (username, vc) in (
                (self.u1.username, "12345678"),
                (self.u1.username, 123),
                ("pouet", "12345678")):
            url = reverse("api-passwordreset-check")
            response = self.client.post(url, {
                "username": username,
                "validation_code": vc
            })
            assert response.status_code == status.HTTP_404_NOT_FOUND

    def test_no_mobilephone(self):
        url = reverse("api-passwordreset-ask")
        response = self.client.post(url, {
            "first_name": self.u2.first_name,
            "last_name": self.u2.last_name,
            "birthdate": datetime.date(2001, 1, 1)
        })
        assert response.status_code == status.HTTP_406_NOT_ACCEPTABLE
        assert response.data["errors"]


class TestDocumentStatistics(AdminTestCase, AbstractTestDocument):
    def setUp(self):
        self.staffuser = self.createClariwebUser("staff", "MED", 50)
        self.staffuser.is_staff = True
        self.staffuser.save()
        super().setUp()

    def test_statistics_permonth_peruser(self):
        self._test_login("staff")
        url = reverse("api-admin-stats-document-month-user")
        response = self.client.get(url)
        assert response.data["count"] >= 12

    def test_statistics_permonth_graph(self):
        self._test_login("staff")
        url = reverse("api-admin-stats-document-month-graph")
        response = self.client.get(url)
        assert len(response.data) >= 12


class AdminUserModifyTestCase(AdminTestCase, BasicTestCase):
    def setUp(self):
        self.staffuser = self.createClariwebUser("staff", "MED", 50)
        self.staffuser.is_staff = True
        self.staffuser.save()
        self.u1 = self.createClariwebUser(
            "user1", "PAT", 1,
            date_naissance=datetime.date(1980, 4, 1),
            address_mobile_phone="+336000000000")
        super().setUp()

    def test_modify_user_active(self):
        self._test_login("staff")
        url = reverse("api-admin-user-modify")
        response = self.client.post(url, {
            "username": "user1", "is_active": False,
        })
        assert response.status_code == status.HTTP_200_OK
        u = BasicUser.objects.get(username="user1")
        assert not u.is_active

    def test_modify_user_password(self):
        self._test_login("staff")
        url = reverse("api-admin-user-modify")
        newpassword = "newpassword"
        assert not self.u1.localpassword
        response = self.client.post(url, {
            "username": "user1", "password": newpassword
        })
        assert response.status_code == status.HTTP_200_OK
        u = BasicUser.objects.get(username="user1")
        assert u.localpassword
        self._test_login("user1", password="newpassword")

    def test_modify_user_password_too_short(self):
        self._test_login("staff")
        url = reverse("api-admin-user-modify")
        assert not self.u1.localpassword
        response = self.client.post(url, {
            "username": "user1", "password": "pan"
        })
        assert response.status_code == status.HTTP_400_BAD_REQUEST

    def test_modify_user_baduser(self):
        self._test_login("staff")
        url = reverse("api-admin-user-modify")
        response = self.client.post(url, {
            "username": "nonexistant.user",
        })
        assert response.status_code == status.HTTP_404_NOT_FOUND

    def test_modify_user_badrequest(self):
        self._test_login("staff")
        url = reverse("api-admin-user-modify")
        response = self.client.post(url, {"unknown": "value"})
        assert response.status_code == status.HTTP_400_BAD_REQUEST


class PaymentRequestTestCase(EmptyCacheAPITestCase, BasicTestCase):
    def setUp(self):
        self.u1 = self.createClariwebUser(
            "pat1", "PAT", 1,
            date_naissance=datetime.date(1980, 4, 1),
            address_mobile_phone="+336000000000",
            debpt=0)
        self.u2 = self.createClariwebUser(
            "pat2", "PAT", 2,
            date_naissance=datetime.date(1950, 1, 1),
            email="pat2@user.fr",
            debpt=1000)
        self.u3 = self.createClariwebUser(
            "pat3", "PAT", 3, first_name="Émmanuel", last_name="àlarùe",
            date_naissance=datetime.date(1930, 1, 1),
            address_mobile_phone="+336000000000",
            email="user3@user.fr",
            debpt=5000)
        self.med1 = self.createClariwebUser("med1", "MED", 10)

    def _test_login(self, username, shouldpass=True, **kwargs):
        data = {"username": username,
                "password": kwargs.get("password", username)}
        url = reverse("api-login")
        response = self.client.post(url, data)
        assert response.status_code == status.HTTP_200_OK
        self.client.credentials(HTTP_X_SESSION_ID=response.data["sessionid"])
        return response

    def test_unlogged(self):
        url = reverse("api-payment-request")
        response = self.client.post(url, {
            "return_url": "http://localhost:1234/"})
        assert response.status_code == status.HTTP_403_FORBIDDEN

    def test_noclariwebuser(self):
        constance.config.SECURITY_IP_ADDRESSES_ADMIN = ["127.0.0.0/8"]
        self.su = BasicUser.objects.create_superuser(
            username="root", password="root")
        self._test_login("root")
        url = reverse("api-payment-request")
        response = self.client.post(url, {
            "return_url": "http://localhost:1234/"})
        assert response.status_code == status.HTTP_404_NOT_FOUND
        assert "errors" in response.data, response.data

    def test_not_patient(self):
        self._test_login("med1")
        url = reverse("api-payment-request")
        response = self.client.post(url, {
            "return_url": "http://localhost:1234/"})
        assert response.status_code == status.HTTP_404_NOT_FOUND
        assert "errors" in response.data, response.data

    def test_no_debpt(self):
        self._test_login("pat1")
        url = reverse("api-payment-request")
        response = self.client.post(url, {
            "return_url": "http://localhost:1234/"})
        assert response.status_code == status.HTTP_404_NOT_FOUND
        assert "errors" in response.data, response.data

    def test_invalid(self):
        self._test_login("pat2")
        url = reverse("api-payment-request")
        response = self.client.post(url)
        assert response.status_code == status.HTTP_400_BAD_REQUEST
        assert "errors" in response.data, response.data

    def _test_good(self, login):
        self._test_login(login)
        url = reverse("api-payment-request")
        response = self.client.post(url, {
            "return_url": "http://localhost:1234/"})
        assert response.status_code == status.HTTP_200_OK
        assert "signature" in response.data, response.data
        assert "vads_trans_id" in response.data, response.data
        assert response.data["vads_amount"] == self.u2.clariwebuser.debpt
        assert "payzen_url" in response.data and response.data["payzen_url"]

    def test_good(self):
        self._test_good("pat2")

    def test_no_first_name(self):
        self.u2.first_name = ''
        self.u2.save()
        self._test_good("pat2")

    def test_no_last_name(self):
        self.u2.last_name = ''
        self.u2.save()
        self._test_good("pat2")

    def test_unconfirmed_email(self):
        assert not self.u1.is_confirmed
        self._test_login("pat2")
        response = self.client.post(reverse("api-payment-request"), {
            "return_url": "http://localhost:1234/"})
        assert response.status_code == status.HTTP_200_OK
        assert response.data["vads_cust_email"] == ''

    def test_confirmed_email(self):
        self.u2.add_unconfirmed_email(self.u2.email)
        self.u2.confirm_email(self.u2.confirmation_key, "pat2@user.fr")
        assert self.u2.is_confirmed
        self._test_login("pat2")
        response = self.client.post(reverse("api-payment-request"), {
            "return_url": "http://localhost:1234/"})
        assert response.status_code == status.HTTP_200_OK
        assert response.data["vads_cust_email"] == self.u2.email

    def test_no_email(self):
        self.u2.email = ''
        self.u2.save()
        self._test_good("pat2")
