# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import constance
from rest_framework import mixins, viewsets, generics, status
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework.permissions import AllowAny
from django.conf import settings
from django.db.models import Q
from django.http import HttpResponse, StreamingHttpResponse
from django.contrib.auth import authenticate, login, logout
from simple_email_confirmation.models import EmailAddress
from brake.decorators import ratelimit
from clariweb.clariweb.models import Document
from clariweb.api.serializers import (
    DocumentSerializer,
    LoginSerializer,
    UserSerializer,
    LoginInvoiceSerializer,
)
from clariweb.api.accesslog import login_success, login_error, document_access
from clariweb.api.filters import SearchFilter
from clariweb.api.pagination import DocumentListPagination


class ListDocumentView(generics.ListAPIView):
    serializer_class = DocumentSerializer
    filter_backends = (SearchFilter, )
    # LC: Don't put other fields because it will kill performances
    # on postgresql with big tables
    # LC: don't use "^" on last_name because of composite last
    # names, like "Le bourdais".
    search_fields = ('patient__last_name^', '^patient__first_name^')
    pagination_class = DocumentListPagination

    def get_queryset(self):
        qs = Document.objects.foruser(self.request.user)
        if self.request.session.get("invoice_only", False):
            qs = qs.filter(is_invoice=True)
        # LC: Select related is counter productive on thoses
        # fields because postgresql planning will do incorrect
        # assumptions
        # qs = qs.select_related("user__user", "medecin",
        #                        "patient", "correspondant")
        qs = qs.order_by(
            'isforeign', '-sil_demande_date', 'sil_demande_number')
        return qs


class ListNotSeenDocumentView(ListDocumentView):
    def get_queryset(self):
        qs = super().get_queryset()
        return qs.exclude(access_log__user__username=self.request.user)


class DocumentDetailView(mixins.RetrieveModelMixin,
                         viewsets.GenericViewSet):
    serializer_class = DocumentSerializer

    def get_queryset(self):
        qs = Document.objects.foruser(self.request.user)
        if self.request.session.get("invoice_only", False):
            qs = qs.filter(is_invoice=True)
        return qs


class DocumentDownload(APIView):
    def render_to_response(self, fileobject, content_type):
        response = StreamingHttpResponse(fileobject.chunks(),
                                         content_type=content_type,
                                         status=status.HTTP_200_OK)
        return response

    def get(self, request, document_pk, filename=None, attachment=True):
        clause = Q()
        if constance.config.DEBPT_MINIMUM_AMOUNT > 0:
            clause = (
                Q(is_invoice=True) |
                Q(unpaid_amount=0) |
                Q(user__debpt__lte=constance.config.DEBPT_MINIMUM_AMOUNT)
            )
        try:
            documents = Document.objects.foruser(self.request.user) \
                .filter(
                    Q(user__kind="PAT") & clause | ~Q(user__kind="PAT"))
            if request.session.get("invoice_only", False):
                documents = documents.filter(is_invoice=True)
            document = documents.get(pk=document_pk)
            datafile = document.datafile
            if not datafile:
                raise ValueError(
                    "document datafile must be present to be downloaded.")
            document_access(request, document)
            response = self.render_to_response(
                datafile,
                document.get_content_type_display())
            if attachment:
                if not filename:
                    disposition = 'attachment;filename="%s_%s.PDF"' % (
                        document.document.replace(" ", "_"),
                        document.sil_demande_number)
                else:
                    disposition = 'attachment'
                response["Content-Disposition"] = disposition
            response["Content-Type"] = "application/pdf"
            response["Content-Length"] = datafile.size
            return response
        except (Document.DoesNotExist, ValueError) as e:
            return HttpResponse("{}".format(e), status=status.HTTP_404_NOT_FOUND)


class DocumentDownloadStream(DocumentDownload):
    def get(self, request, document_pk):
        return super().get(request, document_pk, attachment=False)


class LoginView(APIView):
    permission_classes = (AllowAny, )

    @staticmethod
    @ratelimit(rate='5/min')
    def login_ratelimit(request):
        if getattr(request, 'limited', False):
            return Response({}, status=status.HTTP_429_TOO_MANY_REQUESTS)
        return None

    @staticmethod
    @ratelimit(rate='10/min')
    def autologin_ratelimit(request):
        if getattr(request, 'limited', False):
            return Response({}, status=status.HTTP_429_TOO_MANY_REQUESTS)
        return None

    def _get_logindata(self, request, errors=[]):
        logindata = {
            'isLogged': False,
            'active': False,
            'user': {},
            'errors': errors,
            'setsessionexpiry': False,
            'expiry_time': request.session.get_expiry_age(),
        }
        if request.user.is_authenticated:
            debpt = 0
            debpt_overlimit = False
            userkind = "PAT"
            logindata["isLogged"] = True
            logindata["active"] = True
            logindata["user"] = UserSerializer(request.user).data
            logindata["sessionid"] = request.session.session_key
            clariwebuser = request.user.clariwebuser
            if clariwebuser:
                debpt = clariwebuser.debpt
                debpt_minimum_amount = constance.config.DEBPT_MINIMUM_AMOUNT
                userkind = clariwebuser.kind
                if (debpt_minimum_amount > 0 and
                        clariwebuser.debpt > debpt_minimum_amount):
                    debpt_overlimit = True
            logindata["user"].update({
                "debpt": debpt,
                "debpt_overlimit": debpt_overlimit,
                "kind": userkind,
            })
        return logindata

    def get(self, request):
        """Login using sessions / cookies"""
        resp_status = status.HTTP_401_UNAUTHORIZED
        resp_data = {}
        ratelimit = self.autologin_ratelimit(request)
        if ratelimit is not None:
            return ratelimit
        if constance.config.LOGIN_USING_SESSIONS:
            resp_data = self._get_logindata(request)
            if request.user.is_authenticated:
                resp_status = status.HTTP_200_OK
                login_success(request)
        else:
            logout(request)
        return Response(resp_data, status=resp_status)

    def post(self, request):
        errors = []
        resp_status = status.HTTP_400_BAD_REQUEST
        # We re-authenticate if the user is already logged-in, but
        # re-submit the form
        if (not request.user.is_authenticated or
                request.data.get("username")):
            logout(request)
            if request.data.get("username") and request.data.get("password"):
                ratelimit = self.login_ratelimit(request)
                if ratelimit is not None:
                    return ratelimit
            serializer = LoginSerializer(data=request.data)
            if serializer.is_valid():
                user = authenticate(username=serializer.data['username'],
                                    password=serializer.data['password'],
                                    request=request)
                resp_status = status.HTTP_401_UNAUTHORIZED
                if user and user.is_active:
                    underage_forbidden = constance.config.UNDERAGE_FORBIDDEN
                    if user.is_underaged() and underage_forbidden:
                        resp_status = status.HTTP_403_FORBIDDEN
                    else:
                        login(request, user)
            else:
                errors = serializer.errors
        # Yes login should have modified our "user" we recheck with a if
        logindata = self._get_logindata(request, errors)
        if request.user.is_authenticated:
            resp_status = status.HTTP_200_OK
            login_success(request)
            if request.user.is_using_session_login():
                logindata["setsessionexpiry"] = True
        else:
            if serializer.data.get("username"):
                login_error(request, username=serializer.data['username'])
        return Response(logindata, status=resp_status)


class LoginInvoiceView(LoginView):
    def _get_logindata(self, *args, **kwargs):
        d = super()._get_logindata(*args, **kwargs)
        d["user"]["invoice_only"] = True
        return d

    def post(self, request):
        logout(request)
        errors = []
        serializer = LoginInvoiceSerializer(data=request.data)
        if not serializer.is_valid():
            errors = serializer.errors
            resp_status = status.HTTP_400_BAD_REQUEST
        else:
            ratelimit = self.login_ratelimit(request)
            if ratelimit is not None:
                return ratelimit
            documents = Document.objects.filter(
                is_invoice=True,
                invoice_id=serializer.data["invoice_id"],
                user__kind="PAT",
                user__date_naissance=serializer.data["birthdate"]) \
                .select_related("user__user")[:1]
            user = None
            resp_status = status.HTTP_401_UNAUTHORIZED
            if documents:
                doc = documents[0]
                user = authenticate(username=doc.user.user.username,
                                    birthdate=doc.user.date_naissance,
                                    request=request)
            if user and user.is_active:
                login(request, user)
                request.session["invoice_only"] = True
        if request.user.is_authenticated:
            resp_status = status.HTTP_200_OK
            login_success(request)
        logindata = self._get_logindata(request, errors)
        return Response(logindata, status=resp_status)


class SetSessionExpiryView(APIView):
    def post(self, request):
        resp_status = status.HTTP_403_FORBIDDEN
        resp_data = {}

        expiry_choice = request.data.get("expiry_choice")
        if expiry_choice not in settings.SESSION_LOGIN_EXPIRY_CHOICES.keys():
            resp_status = status.HTTP_400_BAD_REQUEST
        else:
            if request.user.is_using_session_login() or expiry_choice == 1:
                resp_status = status.HTTP_200_OK
                expiry_time = \
                    settings.SESSION_LOGIN_EXPIRY_CHOICES.get(expiry_choice)
                request.session.set_expiry(expiry_time)
                resp_data = {"expiry_time": expiry_time}
            else:
                resp_status = status.HTTP_403_FORBIDDEN
        return Response(resp_data, status=resp_status)


class LogoutView(APIView):
    permission_classes = (AllowAny, )

    def get(self, request):
        logout(request)
        return Response({}, status=status.HTTP_200_OK)


class EmailConfirmView(APIView):
    permission_classes = (AllowAny, )

    def post(self, request):
        key = request.data.get("key")
        if key and len(key) == 12:
            try:
                address = EmailAddress.objects.get(key=key)
                if address.is_key_expired:
                    return Response({"expired": True},
                                    status=status.HTTP_404_NOT_FOUND)
                EmailAddress.objects.confirm(key, address.user)
                resp = {
                    "user": {
                        "email": address.email,
                        "full_name": address.user.get_full_name(),
                    }
                }
                return Response(resp, status=status.HTTP_200_OK)
            except EmailAddress.DoesNotExist:
                pass
        return Response({"error": "Clef incorrecte"},
                        status=status.HTTP_400_BAD_REQUEST)


class EmailConfirmResendView(APIView):
    @staticmethod
    @ratelimit(rate='5/day')
    def send_email_confirmation_ratelimit(request):
        if getattr(request, 'limited', False):
            return Response({}, status=status.HTTP_429_TOO_MANY_REQUESTS)
        return None

    def get(self, request):
        ratelimited = self.send_email_confirmation_ratelimit(request)
        if ratelimited is not None:
            return ratelimited
        request.user.reset_email_confirmation(request.user.email)
        request.user.send_email_confirmation()
        return Response({}, status=status.HTTP_200_OK)


class CustomSettingsView(APIView):
    permission_classes = (AllowAny, )

    def get(self, request):
        data = {}
        keys = ("DEBPT_MINIMUM_AMOUNT", "CLINIC_ENABLED")
        for key in dir(constance.config):
            if key.startswith("CUSTOM") or key in keys:
                data[key] = getattr(constance.config, key)
        return Response(data, status=status.HTTP_200_OK)
