# -*- coding: utf-8 -*-

from __future__ import absolute_import

import constance
import requests
from requests.auth import AuthBase
from django.core.mail import EmailMessage
from django.core import management
from django.conf import settings
from django.db import transaction
from celery.utils.log import get_task_logger
from clariweb.clariweb.models import (
    Notification, ClarilabPaymentResponse, PaymentResponse)
from clariweb.clariweb.smscount import SMSCounter
from clariweb.celery import app
from . import ovh_sms_api


logger = get_task_logger('backend')


@app.task
def update_notification_status(*args, **kwargs):
    notification = Notification.objects.get(pk=kwargs["notification_id"])
    notification.status = "SENT"
    notification.save(update_fields=["status"])


@app.task(bind=True)
def update_notification_status_error(self, *args, **kwargs):
    notification = Notification.objects.get(pk=kwargs["notification_id"])
    if self.request.retries >= self.max_retries:
        logger.error("Send error: max retry exceeded")
    notification.status = "ERROR"
    notification.save(update_fields=["status"])


@app.task(bind=True, max_retries=4, default_retry_delay=3600)
def _send_sms_notification(self, notification_id, numbers, message, **kwargs):
    logger.debug("send_sms_notification id=%s", notification_id)
    try:
        ovh_sms_api.send_sms(numbers, message, **kwargs)
    except Exception as exc:
        raise self.retry(exc=exc)


@app.task(bind=True, max_retries=4, default_retry_delay=3600)
def _send_email_notification(self, notification_id, emails, subject, body,
                             **kwargs):
    if not isinstance(emails, (list, tuple)):
        emails = [emails, ]
    logger.debug("send_email_notification id=%s emails=%s" % (
                 notification_id, emails))
    try:
        mail = EmailMessage(subject.replace('\n', ''),
                            body,
                            constance.config.EMAIL_FROM,
                            emails)
        mail.send()
    except Exception as exc:
        # TODO: Use autoretry_for
        raise self.retry(exc=exc)


def send_email_notification(*args, **kwargs):
    return _send_email_notification.apply_async(
        args=args, kwargs=kwargs,
        link=update_notification_status.s(*args, **kwargs),
        link_error=update_notification_status_error.s(*args, **kwargs))


def send_sms_notification(*args, **kwargs):
    return _send_sms_notification.apply_async(
        args=args, kwargs=kwargs,
        link=update_notification_status.s(*args, **kwargs),
        link_error=update_notification_status_error.s(*args, **kwargs))


@app.task(autoretry_for=(ovh_sms_api.RetryException,),
          retry_kwargs={'max_retries': 4, 'default_retry_delay': 60})
def update_sms_quota():
    quotaleft = ovh_sms_api.get_quota_left()
    logger.debug("SMS quota left updated: %s", quotaleft)
    SMSCounter.set_quota_left(quotaleft)
    return quotaleft


@app.task(time_limit=60 * 60 * 6)
def periodic_deleteorphaned():
    """This task is LOOOOONG. Specifiy that it can take 6 hours."""
    management.call_command('purge_orphaned')


@app.task
def periodic_clear_sessions():
    management.call_command('clearsessions')


@app.task
def periodic_clear_passwordreset():
    management.call_command('cleanpasswordreset')


@app.task
def periodic_refreshstatsviews():
    management.call_command('refreshstatsviews')


@app.task
def periodoc_clean_paymentrequests():
    management.call_command('clean_paymentrequests')


@app.task
def periodic_cleansessions():
    management.call_command('clearsessions')


class TokenAuth(AuthBase):
    def __init__(self, token):
        self.token = token

    def __call__(self, r):
        r.headers["Authorization"] = "Token %s" % (self.token, )
        return r


@app.task(bind=True, max_retries=10, default_retry_delay=300,
          autoretry_for=(requests.ConnectionError,))
def clarilab_reglement(self, payment_response_id):
    @transaction.atomic
    def clarilab_response(msg, success=False):
        try:
            prf = ClarilabPaymentResponse.objects.get(
                response=payment_response)
        except ClarilabPaymentResponse.DoesNotExist:
            prf = ClarilabPaymentResponse(response=payment_response)
        prf.reason = msg
        prf.success = success
        prf.save()

    payment_response = PaymentResponse.objects.get(pk=payment_response_id)
    logger.info("ask clarilab reglement for payment_response %s",
                payment_response.pk)
    user = payment_response.user
    assert(user is not None)
    clariwebuser_id = user.clariwebuser.sil_pk
    try:
        r = requests.post(
            "%s/reglement" % (settings.CLARILAB_API_URL,),
            data={
                "amount_cents": payment_response.vads_amount,
                "transaction_id": payment_response.vads_trans_id,
                "user_id": clariwebuser_id,
            },
            auth=TokenAuth(settings.CLARILAB_API_TOKEN))
        # All other errors are non fatal errors we don't expect.
        if r.status_code not in (200, 500):
            msg = "%s error from clarilab: %s" % (r.status_code, r.text)
            logger.warning(msg)
            clarilab_response(msg, success=False)
        elif r.status_code != 200:
            # retries starts at 0
            if self.request.retries > (self.max_retries - 2):
                logger.error(
                    "clarilab payment response error: retried too many times (%s)",  # noqa
                    self.request.retries)
                clarilab_response("too many tries", success=False)
            else:
                msg = "%s error from clarilab: %s" % (r.status_code, r.text)
                logger.warning(msg)
                raise self.retry()
    except Exception:
        logger.exception("unknown exception occured")
        # raise self.retry(exc=exc)
    else:
        if r.status_code == 200:
            logger.info("clarilab payment response accepted: %s", r.json())
            clarilab_response("{}".format(r.json()), success=True)
        else:
            logger.error("clarilab payment unknown error: %s", r.status_code)
            clarilab_response("unknown return code %s" % (r.status_code,),
                              success=False)
