# -*- coding: utf-8 -*-

import datetime
import httpretty
import pytest
from pytest import raises
from django.utils import timezone
from django.test import TestCase, override_settings
from django.conf import settings
from mock import patch
from celery.exceptions import Retry
from clariweb.clariweb.models import (
    Notification, PaymentRequest, PaymentResponse, ClarilabPaymentResponse)
from clariweb.clariweb.test_clariweb import ClariwebTestCaseBase
from clariweb.backend import tasks
from clariweb.backend.ovh_sms_api import RetryException


@pytest.fixture(scope='session')
def celery_config():
    return {
        'broker_url': 'redis://localhost:6379/1',
        'result_backend': 'redis://localhost:6379/1',
    }


class BackendTasksTestCase(ClariwebTestCaseBase, TestCase):
    def setUp(self):
        self.u1 = self.createClariwebUser("user1", "MED", 10, civilite="Dr",
                                          email="user1@test.resulabo.fr",
                                          address_mobile_phone="+33600000000")
        self.p1 = self.createClariwebUser("pat1", "PAT", 20, civilite="Mme",
                                          email="pat1@test.resulabo.fr",
                                          address_mobile_phone="+33600000000",
                                          debpt=4400)

    @patch('clariweb.backend.ovh_sms_api.get_quota_left')
    def test_update_sms_quota(self, get_quota_left):
        get_quota_left.return_value = 10
        tasks.update_sms_quota()
        get_quota_left.assert_called_once()

    @patch('clariweb.backend.ovh_sms_api.get_quota_left')
    @patch('clariweb.backend.tasks.update_sms_quota.retry')
    def test_update_sms_quota_fail(self, update_sms_quota_retry,
                                   get_quota_left):
        get_quota_left.side_effect = RetryException("Error")
        update_sms_quota_retry.side_effect = Retry()
        with raises(Retry):
            tasks.update_sms_quota()
        get_quota_left.assert_called()

    @override_settings(
        task_eager_propagates=True,
        task_always_eager=True,
        broker_url='memory://',
        backend='memory')
    def test_send_email_notification(self):
        notification = Notification.objects.create(
            user=self.u1.clariwebuser, status="WAITING",
            email=self.u1.email)
        tasks.send_email_notification(
            notification_id=notification.pk,
            emails=[notification.email, ],
            subject="Test email",
            body="Test body")
        notification = Notification.objects.get(pk=notification.pk)
        assert notification.status == "SENT"

    @override_settings(
        task_eager_propagates=True,
        task_always_eager=True,
        broker_url='memory://',
        backend='memory')
    def test_send_sms_notification(self):
        notification = Notification.objects.create(
            user=self.u1.clariwebuser, status="WAITING",
            sms=self.u1.clariwebuser.address_mobile_phone)
        tasks.send_sms_notification(
            notification_id=notification.pk,
            numbers=[notification.sms],
            message="Test SMS message")
        notification = Notification.objects.get(pk=notification.pk)
        assert notification.status == "SENT"

    def test_deleteorphaned(self):
        tasks.periodic_deleteorphaned.delay()

    def test_clear_sessions(self):
        tasks.periodic_clear_sessions.delay()

    def test_refresh_views(self):
        tasks.periodic_refreshstatsviews()

    @override_settings(
        task_eager_propagates=True,
        task_always_eager=True,
        broker_url='memory://',
        backend='memory')
    def test_clean_paymentrequests(self):
        pr = PaymentRequest(vads_amount=100)
        pr.save()
        pr.date_insert = timezone.now() - datetime.timedelta(seconds=800)
        pr.save()
        assert PaymentRequest.objects.count() == 1
        tasks.periodoc_clean_paymentrequests()
        assert PaymentRequest.objects.count() == 0


@override_settings(task_eager_propagates=True, task_always_eager=True,
                   broker_url='memory://', backend='memory')
class BackendPaymentResponseTestCase(ClariwebTestCaseBase, TestCase):
    def setUp(self):
        self.p1 = self.createClariwebUser("pat1", "PAT", 20, civilite="Mme",
                                          email="pat1@test.resulabo.fr",
                                          address_mobile_phone="+33600000000",
                                          debpt=4400)

    def _add_payment(self, user, **kw):
        payment_request = PaymentRequest(
            vads_amount=kw.get("amount", user.clariwebuser.debpt),
            user=user,
            vads_cust_id=user.pk,
            vads_cust_email=user.email if user.is_confirmed else '',
            vads_cust_title=user.clariwebuser.civilite,
            vads_cust_last_name=user.last_name,
            vads_cust_first_name=user.first_name,
            vads_capture_delay=0,
            vads_payment_config='SINGLE',
            vads_return_mode='GET',
            vads_validation_mode=0,
        )
        payment_request.save()
        return payment_request

    def _add_response(self, payment_request, success=False):
        payment_response = PaymentResponse(
            vads_amount=payment_request.vads_amount,
            vads_result="00" if success else "02",
            vads_trans_date=payment_request.vads_trans_date,
            vads_trans_id=payment_request.vads_trans_id,
            vads_site_id=payment_request.vads_site_id,
        )
        payment_response.save()
        payment_request.pzresponse = payment_response
        payment_request.save()
        return payment_response

    @httpretty.activate
    def test_response_good(self):
        uri = "%s/reglement" % (settings.CLARILAB_API_URL,)
        httpretty.register_uri(httpretty.POST, uri,
                               body='{}',
                               content_type="application/json",
                               status=200)
        req = self._add_payment(self.p1)
        self._add_response(req, success=True)
        assert ClarilabPaymentResponse.objects.errors.count() == 0
        assert ClarilabPaymentResponse.objects.success.count() == 1

    @httpretty.activate
    def test_response_fatal(self):
        httpretty.register_uri(
            httpretty.POST, "%s/reglement" % (settings.CLARILAB_API_URL,),
            responses=[
                httpretty.Response(body="{}", status=404),
                httpretty.Response(body="{}", status=200),
            ]
        )
        req = self._add_payment(self.p1)
        self._add_response(req, success=True)
        assert ClarilabPaymentResponse.objects.errors.count() == 1
        assert ClarilabPaymentResponse.objects.success.count() == 0

    @httpretty.activate
    def test_response_recoverable(self):
        httpretty.register_uri(
            httpretty.POST, "%s/reglement" % (settings.CLARILAB_API_URL,),
            responses=[
                httpretty.Response(body="{}", status=500),
                httpretty.Response(body="{}", status=200),
            ]
        )
        req = self._add_payment(self.p1)
        self._add_response(req, success=True)
        assert ClarilabPaymentResponse.objects.errors.count() == 0
        assert ClarilabPaymentResponse.objects.success.count() == 1

    @httpretty.activate
    def test_response_too_many_tries(self):
        responses = [httpretty.Response(body="{}", status=500)] * 10
        responses.append(httpretty.Response(body="{}", status=200))
        httpretty.register_uri(
            httpretty.POST, "%s/reglement" % (settings.CLARILAB_API_URL,),
            responses=responses)
        req = self._add_payment(self.p1)
        self._add_response(req, success=True)
        assert ClarilabPaymentResponse.objects.errors.count() == 1
        assert ClarilabPaymentResponse.objects.success.count() == 0
