# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import io
import os
import datetime
import constance
from django.utils import timezone
from django.conf import settings
from django.core.cache import cache
from django.core.files import File
from django.test import TestCase, override_settings
from django.core import mail
from .models import (
    BasicUser,
    ClariwebUser,
    Document,
    Notification,
    PasswordReset,
    AccessLog,
    DocumentAccessLog,
)


class ClariwebTestCaseBase(object):
    def createClariwebUser(self, username, kind, sil_pk, **kwargs):
        user = BasicUser.objects.create_user(**{
            "username": username,
            "password": username,
            "first_name": username,
            "last_name": username,
            "email": kwargs.get("email"),
        })
        clariwebuser = ClariwebUser(**{
            "sil_pk": sil_pk,
            "kind": kind,
            "group": kwargs.get("group", ""),
            "group_clinic": kwargs.get("group_clinic", ""),
            "civilite": kwargs.get("civilite", "Mr"),
            "address_mobile_phone": kwargs.get("address_mobile_phone", "")
        })
        clariwebuser.save()
        user.clariwebuser = clariwebuser
        user.save()
        return user

    def createDocument(self, user, patient, demande_number, **kwargs):
        self.sil_next_pk = getattr(self, "sil_next_pk", 0) + 1
        fpath = kwargs.get("fpath", None)
        if fpath is None:
            fpath = self.test_fpath
        doc = Document.objects.create(**{
            "sil_document_pk": self.sil_next_pk,
            "sil_demande_pk": int(demande_number),
            "sil_demande_number": str(demande_number),
            "sil_demande_date": kwargs.get("sil_demande_date", self.now),
            "document": kwargs.get("document", "Compte-Rendu PDF"),
            "user": user.clariwebuser,
            "datafile": fpath,
            "patient": patient,
            "medecin": kwargs.get("medecin", None),
            "correspondant": kwargs.get("correspondant", None),
            "notify_sms": kwargs.get("notify_sms", False),
            "notify_email": kwargs.get("notify_email", False),
        })
        return doc

    def setUp(self):
        self.test_fpath = os.path.join(settings.BASE_DIR, "tests", "test1.pdf")
        self.su = BasicUser.objects.create_superuser(
            username="root", password="root")
        self.user = BasicUser.objects.create_user(
            username="test1", password="test1")

        self.med1 = self.createClariwebUser("med1", "MED", 10, civilite="Dr",
                                            email="med1@analyses.fr")
        self.med2 = self.createClariwebUser("med2", "MED", 11, civilite="Dr",
                                            group="gr1",
                                            email="med2@analyses.fr")
        self.med3 = self.createClariwebUser("med3", "MED", 12, civilite="Dr",
                                            group="gr1")
        self.corr1 = self.createClariwebUser("cor1", "COR", 20, civilite="",
                                             group="gr1")
        self.corr2 = self.createClariwebUser("cor2", "COR", 21, civilite="",
                                             group="gr2")
        self.prel1 = self.createClariwebUser("prel1", "PREL", 41, civilite="",
                                             group="gr3")
        self.prel2 = self.createClariwebUser("prel2", "PREL", 42, civilite="",
                                             group="gr3")
        self.pat1 = self.createClariwebUser(
            "pat1", "PAT", 30,
            email="pat1@analyses.fr",
            address_mobile_phone="+33606060606")
        self.pat2 = self.createClariwebUser(
            "pat2", "PAT", 31, civilite="Mme",
            address_mobile_phone="+33606060607")
        self.now = timezone.now()
        super().setUp()


class ClariwebUserLoginSesssionTestCase(ClariwebTestCaseBase, TestCase):
    def test_user_session_login(self):
        assert self.med1.clariwebuser.kind == "MED"
        assert self.med1.is_using_session_login()
        assert self.corr1.is_using_session_login()
        assert not self.pat1.is_using_session_login()


class UserEmailTestCase(ClariwebTestCaseBase, TestCase):
    def test_send_email(self):
        mail.outbox = []
        self.med1.email_user("Test email", "test body")
        assert len(mail.outbox) == 1

    def test_send_mail_newuser(self):
        mail.outbox = []
        user = self.createClariwebUser("pat3", "PAT", 32, civilite="Mr")
        assert len(mail.outbox) == 0
        user.email = "pat3@analyses.fr"
        user.save(update_fields=["email"])
        assert self.med1.email in self.med1.get_unconfirmed_emails()

    def test_existing_user_create_email(self):
        mail.outbox = []
        self.med3.email = "med3@modified.fr"
        self.med3.save()
        assert len(mail.outbox) == 0
        assert self.med3.email in self.med3.get_unconfirmed_emails()

    def test_send_mail_existing_user_modify_email(self):
        mail.outbox = []
        self.med1.email = "med1@modified.fr"
        self.med1.save()
        assert len(mail.outbox) == 0
        assert self.med1.email in self.med1.get_unconfirmed_emails()

    def test_email_confirmation_key_length(self):
        user = self.createClariwebUser("pat3", "PAT", 32, civilite="Mr",
                                       email="test@test.test")
        # webui check for this key length.
        user.add_unconfirmed_email(user.email)
        assert len(user.confirmation_key) == 12

    def test_send_email_confirmation(self):
        mail.outbox = []
        user = self.createClariwebUser("foo", "PAT", 32, civilite="Mr",
                                       email="foo@foo.bar")
        assert len(mail.outbox) == 0
        doc = Document(sil_document_pk=1,
                       sil_demande_pk=1,
                       sil_demande_number="1",
                       sil_demande_date=self.now,
                       content_type="application/pdf",
                       document="Compte-Rendu PDF",
                       datafile=File(io.StringIO("{}".format("test"))),
                       user=user.clariwebuser,
                       patient=user.to_actor(),
                       medecin=self.med2.to_actor())
        doc.save()
        assert user.email_address_set.filter(
            email=user.email, email_sent=True).exists()
        assert len(mail.outbox) == 1
        message = mail.outbox[0]
        # Test subject mail
        assert (message.subject ==
                "Confirmation de création de compte")
        # Confirmation key must be present in the mail body
        assert user.confirmation_key in message.body


class DocumentTestCase(ClariwebTestCaseBase, TestCase):
    def test_document_permissions_su(self):
        doc = Document(sil_document_pk=1,
                       sil_demande_pk=1,
                       sil_demande_number="1",
                       sil_demande_date=self.now,
                       content_type="application/pdf",
                       document="Compte-Rendu",
                       datafile=File(io.StringIO("{}".format("test"))),
                       user=self.pat1.clariwebuser,
                       patient=self.pat1.to_actor(),
                       medecin=self.med2.to_actor())
        doc.save()
        assert len(Document.objects.foruser(self.su)) == 1

    def test_document_permissions_nobody(self):
        doc = Document(sil_document_pk=1,
                       sil_demande_pk=1,
                       sil_demande_number="1",
                       sil_demande_date=self.now,
                       content_type="application/pdf",
                       document="Compte-Rendu",
                       datafile=File(io.StringIO("{}".format("test"))),
                       user=self.pat1.clariwebuser,
                       patient=self.pat1.to_actor(),
                       medecin=self.med2.to_actor())
        doc.save()
        assert len(Document.objects.foruser(self.user)) == 0

    def test_document_permissions_patient(self):
        doc = Document(sil_document_pk=1,
                       sil_demande_pk=1,
                       sil_demande_number="1",
                       sil_demande_date=self.now,
                       content_type="application/pdf",
                       document="Compte-Rendu",
                       datafile=File(io.StringIO("{}".format("test"))),
                       user=self.pat1.clariwebuser,
                       patient=self.pat1.to_actor(),
                       medecin=self.med2.to_actor())
        doc.save()
        assert len(Document.objects.foruser(self.pat1)) == 1
        assert len(Document.objects.foruser(self.pat2)) == 0
        assert len(Document.objects.foruser(self.med1)) == 0
        assert len(Document.objects.foruser(self.med2)) == 0
        assert len(Document.objects.foruser(self.med3)) == 0
        assert len(Document.objects.foruser(self.corr1)) == 0
        assert len(Document.objects.foruser(self.corr2)) == 0

    def test_document_permissions_medecin(self):
        doc = Document(sil_document_pk=1,
                       sil_demande_pk=1,
                       sil_demande_number="1",
                       sil_demande_date=self.now,
                       content_type="application/pdf",
                       document="Compte-Rendu",
                       datafile=File(io.StringIO("{}".format("test"))),
                       user=self.med1.clariwebuser,
                       medecin=self.med1.to_actor(),
                       patient=self.pat2.to_actor())
        doc.save()
        assert len(Document.objects.foruser(self.pat1)) == 0
        assert len(Document.objects.foruser(self.pat2)) == 0
        assert len(Document.objects.foruser(self.med1)) == 1
        assert len(Document.objects.foruser(self.med2)) == 0
        assert len(Document.objects.foruser(self.med3)) == 0
        assert len(Document.objects.foruser(self.corr1)) == 0
        assert len(Document.objects.foruser(self.corr2)) == 0

    def test_document_patient_permissions_medecin_group(self):
        doc = Document(sil_document_pk=1,
                       sil_demande_pk=1,
                       sil_demande_number="1",
                       sil_demande_date=self.now,
                       content_type="application/pdf",
                       document="Compte-Rendu",
                       datafile=File(io.StringIO("{}".format("test"))),
                       user=self.pat1.clariwebuser,
                       medecin=self.med2.to_actor(),
                       patient=self.pat1.to_actor())
        doc.save()
        assert len(Document.objects.foruser(self.pat1)) == 1
        assert len(Document.objects.foruser(self.pat2)) == 0
        assert len(Document.objects.foruser(self.med1)) == 0
        assert len(Document.objects.foruser(self.med2)) == 0
        assert len(Document.objects.foruser(self.med3)) == 0
        assert len(Document.objects.foruser(self.corr1)) == 0
        assert len(Document.objects.foruser(self.corr2)) == 0

    def test_document_patient_permissions_medecin_correspondant_group(self):
        doc = Document(sil_document_pk=1,
                       sil_demande_pk=1,
                       sil_demande_number="1",
                       sil_demande_date=self.now,
                       content_type="application/pdf",
                       document="Compte-Rendu",
                       datafile=File(io.StringIO("{}".format("test"))),
                       user=self.med3.clariwebuser,
                       medecin=self.med2.to_actor(),
                       patient=self.pat1.to_actor())
        doc.save()
        assert len(Document.objects.foruser(self.pat1)) == 0
        assert len(Document.objects.foruser(self.pat2)) == 0
        assert len(Document.objects.foruser(self.med1)) == 0
        assert len(Document.objects.foruser(self.med2)) == 1
        assert len(Document.objects.foruser(self.med3)) == 1
        assert len(Document.objects.foruser(self.corr1)) == 1
        assert len(Document.objects.foruser(self.corr2)) == 0

    def test_doc_patient_perms_correspondant_group_clinic(self):
        self.corr2.clariwebuser.group_clinic = "clin1"
        self.corr2.clariwebuser.save()
        self.med1.clariwebuser.group_clinic = "clin1"
        self.med1.clariwebuser.save()
        self.corr3 = self.createClariwebUser("cor3", "COR", 122, civilite="",
                                             group="", group_clinic="clin1")
        assert(self.corr3.clariwebuser.group_clinic ==
               self.corr2.clariwebuser.group_clinic)
        doc = Document(sil_document_pk=1,
                       sil_demande_pk=1,
                       sil_demande_number="1",
                       sil_demande_date=self.now,
                       content_type="application/pdf",
                       document="Compte-Rendu",
                       datafile=File(io.StringIO("{}".format("test"))),
                       user=self.corr2.clariwebuser,
                       medecin=self.med2.to_actor(),
                       patient=self.pat1.to_actor())
        doc.save()
        assert len(Document.objects.foruser(self.pat1)) == 0
        assert len(Document.objects.foruser(self.pat2)) == 0
        assert len(Document.objects.foruser(self.med1)) == 0
        assert len(Document.objects.foruser(self.med2)) == 0
        assert len(Document.objects.foruser(self.med3)) == 0
        assert len(Document.objects.foruser(self.corr1)) == 0
        assert len(Document.objects.foruser(self.corr2)) == 1
        assert len(Document.objects.foruser(self.corr3)) == 1

    def test_document_patient_permissions_preleveur_group(self):
        doc = Document(sil_document_pk=1,
                       sil_demande_pk=1,
                       sil_demande_number="1",
                       sil_demande_date=self.now,
                       content_type="application/pdf",
                       document="Compte-Rendu",
                       datafile=File(io.StringIO("{}".format("test"))),
                       user=self.prel1.clariwebuser,
                       medecin=self.med2.to_actor(),
                       patient=self.pat1.to_actor())
        doc.save()
        assert len(Document.objects.foruser(self.pat1)) == 0
        assert len(Document.objects.foruser(self.pat2)) == 0
        assert len(Document.objects.foruser(self.med1)) == 0
        assert len(Document.objects.foruser(self.med2)) == 0
        assert len(Document.objects.foruser(self.med3)) == 0
        assert len(Document.objects.foruser(self.corr1)) == 0
        assert len(Document.objects.foruser(self.corr2)) == 0
        assert len(Document.objects.foruser(self.prel1)) == 1
        assert len(Document.objects.foruser(self.prel2)) == 1

    def test_document_permissions_preleveur_nogroup(self):
        prel3 = self.createClariwebUser("prel3", "PREL", 45, civilite="",
                                        group="")
        prel4 = self.createClariwebUser("prel4", "PREL", 46, civilite="",
                                        group="")
        doc = Document(sil_document_pk=1,
                       sil_demande_pk=1,
                       sil_demande_number="1",
                       sil_demande_date=self.now,
                       content_type="application/pdf",
                       document="Compte-Rendu",
                       datafile=File(io.StringIO("{}".format("test"))),
                       user=prel3.clariwebuser,
                       medecin=self.med2.to_actor(),
                       patient=self.pat1.to_actor())
        doc.save()
        assert len(Document.objects.foruser(self.pat1)) == 0
        assert len(Document.objects.foruser(self.pat2)) == 0
        assert len(Document.objects.foruser(self.med1)) == 0
        assert len(Document.objects.foruser(self.med2)) == 0
        assert len(Document.objects.foruser(self.med3)) == 0
        assert len(Document.objects.foruser(self.corr1)) == 0
        assert len(Document.objects.foruser(self.corr2)) == 0
        assert len(Document.objects.foruser(self.prel1)) == 0
        assert len(Document.objects.foruser(self.prel2)) == 0
        assert len(Document.objects.foruser(prel3)) == 1
        assert len(Document.objects.foruser(prel4)) == 0

    def test_document_permissions_medecin_group_forpatient(self):
        doc = Document(sil_document_pk=1,
                       sil_demande_pk=1,
                       sil_demande_number="1",
                       sil_demande_date=self.now,
                       content_type="application/pdf",
                       document="Compte-Rendu",
                       datafile=File(io.StringIO("{}".format("test"))),
                       user=self.pat1.clariwebuser,
                       medecin=self.med2.to_actor(),
                       patient=self.pat1.to_actor())
        doc.save()
        assert len(Document.objects.foruser(self.pat1)) == 1
        assert len(Document.objects.foruser(self.pat2)) == 0
        assert len(Document.objects.foruser(self.med1)) == 0
        assert len(Document.objects.foruser(self.med2)) == 0
        assert len(Document.objects.foruser(self.med3)) == 0
        assert len(Document.objects.foruser(self.corr1)) == 0
        assert len(Document.objects.foruser(self.corr2)) == 0

    def test_document_properties(self):
        Document(sil_document_pk=1,
                 sil_demande_pk=1,
                 sil_demande_number="a1",
                 sil_demande_date=self.now,
                 content_type="application/pdf",
                 document="Compte Rendu",
                 datafile=File(io.StringIO("{}".format("test"))),
                 user=self.pat1.clariwebuser,
                 patient=self.pat1.to_actor(),
                 medecin=self.med2.to_actor(),
                 analyses="VS TP POT").save()
        doc = Document.objects.get(sil_document_pk=1)
        assert doc.document_slug == "compte_rendu"
        assert "{}".format(doc) == "a1 Compte Rendu"
        assert doc.analyses == "VS TP POT"


@override_settings(task_eager_propagates=True, task_always_eager=True,
                   broker_url='memory://', backend='memory')
class TestNotifications(ClariwebTestCaseBase, TestCase):
    def setUp(self):
        import shutil
        tests_file_path = os.path.join(settings.DOCUMENTS_ROOT, "tests")
        dest_test_filepath = os.path.join(tests_file_path, "test1.pdf")
        if not os.path.isdir(tests_file_path):
            os.mkdir(tests_file_path)
            shutil.copy(os.path.join(settings.BASE_DIR, "tests", "test1.pdf"),
                        dest_test_filepath)
        self.test_fpath = dest_test_filepath
        self.now = timezone.now()
        settings.SMS_OVH_HTTP2SMS = "http://localhost:8002/test/ovh_http2sms"
        self.med1 = self.createClariwebUser(
            "med1", "MED", 10, civilite="Dr",
            email="med@medecin.fr",
            address_mobile_phone="+33622222222")
        self.prel1 = self.createClariwebUser(
            "prel1", "PREL", 11, civilite="Prel",
            email="prel@preleveur.fr",
            address_mobile_phone="+33633333333")
        self.pat = self.createClariwebUser("pat", "PAT", 20,
                                           email="pat@patient.fr",
                                           address_mobile_phone="+33611111111")
        constance.config.SMS_NOTIFICATION_PATIENT = True
        constance.config.SMS_NOTIFICATION_MEDECIN = True
        constance.config.SMS_NOTIFICATION_CORRESPONDANT = True
        constance.config.SMS_NOTIFICATION_PRELEVEUR = True

    def tearDown(self):
        cache.clear()

    def test_document_zero_notification(self):
        self.createDocument(self.pat, self.pat.to_actor(), 42,
                            notify_sms=False, notify_email=False)
        assert len(Notification.objects.all()) == 0

    def test_email_notification_without_email_confirmed(self):
        constance.config.EMAIL_NOTIFICATION_UNCONFIRMED = False
        assert not self.pat.is_confirmed
        self.createDocument(self.pat, self.pat.to_actor(), 42,
                            notify_sms=False, notify_email=True)
        notifications = Notification.objects.filter(user=self.pat.clariwebuser)
        assert len(notifications) == 0

    def test_email_notification_without_email_confirmed_but_shouldpass(self):
        constance.config.EMAIL_NOTIFICATION_UNCONFIRMED = True
        assert not self.pat.is_confirmed
        self.createDocument(self.pat, self.pat.to_actor(), 42,
                            notify_sms=False, notify_email=True)
        notifications = Notification.objects.filter(user=self.pat.clariwebuser)
        assert len(notifications) == 1

    def test_document_email_notification(self):
        self.pat.add_unconfirmed_email(self.pat.email)
        self.pat.confirm_email(self.pat.confirmation_key, self.pat.email)
        assert self.pat.is_confirmed
        self.createDocument(self.pat, self.pat.to_actor(), 42,
                            notify_sms=False, notify_email=True)
        notifications = Notification.objects.filter(user=self.pat.clariwebuser)
        assert len(notifications) == 1
        notif = notifications[0]
        assert notif.email == self.pat.email
        assert notif.status == "SENT"

    def test_document_email_multiple_notifications(self):
        constance.config.NOTIFICATION_SAMEUSER_MINIMUM_DELAY = 1
        self.pat.add_unconfirmed_email(self.pat.email)
        self.pat.confirm_email(self.pat.confirmation_key, self.pat.email)
        assert self.pat.is_confirmed
        self.createDocument(self.pat, self.pat.to_actor(), 42,
                            notify_sms=False, notify_email=True)
        notifications = Notification.objects.filter(user=self.pat.clariwebuser)
        assert len(notifications) == 1
        self.createDocument(self.pat, self.pat.to_actor(), 43,
                            notify_sms=False, notify_email=True)
        notifications = Notification.objects.filter(user=self.pat.clariwebuser)
        assert len(notifications) == 1

    def test_document_email_multiple_notifications_post_delay(self):
        constance.config.NOTIFICATION_SAMEUSER_MINIMUM_DELAY = 1
        self.pat.add_unconfirmed_email(self.pat.email)
        self.pat.confirm_email(self.pat.confirmation_key, self.pat.email)
        assert self.pat.is_confirmed
        self.createDocument(self.pat, self.pat.to_actor(), 42,
                            notify_sms=True, notify_email=True)
        notifications = Notification.objects.filter(user=self.pat.clariwebuser)
        assert len(notifications) == 2

    def test_document_email_multiple_notifications_email_and_sms(self):
        constance.config.NOTIFICATION_SAMEUSER_MINIMUM_DELAY = 1
        self.pat.add_unconfirmed_email(self.pat.email)
        self.pat.confirm_email(self.pat.confirmation_key, self.pat.email)
        assert self.pat.is_confirmed
        self.createDocument(self.pat, self.pat.to_actor(), 42,
                            notify_sms=False, notify_email=True)
        notifications = Notification.objects.filter(user=self.pat.clariwebuser)
        assert len(notifications) == 1
        self.createDocument(self.pat, self.pat.to_actor(), 43,
                            notify_sms=False, notify_email=True)
        notifications = Notification.objects.filter(user=self.pat.clariwebuser)
        assert len(notifications) == 1

    def test_document_email_notification_med(self):
        self.med1.add_unconfirmed_email(self.med1.email)
        self.med1.confirm_email(self.med1.confirmation_key, self.med1.email)
        assert self.med1.is_confirmed
        self.createDocument(self.med1, self.pat.to_actor(), 42,
                            notify_sms=False, notify_email=True)
        notifications = Notification.objects.filter(
            user=self.med1.clariwebuser)
        assert len(notifications) == 1
        notif = notifications[0]
        assert notif.email == self.med1.email
        assert notif.status == "SENT"

    def test_document_sms_notification_pat(self):
        self.pat.add_unconfirmed_email(self.pat.email)
        self.pat.confirm_email(self.pat.confirmation_key, self.pat.email)
        assert self.pat.is_confirmed
        assert constance.config.SMS_NOTIFICATION_PATIENT
        self.createDocument(self.pat, self.pat.to_actor(), 42,
                            notify_sms=True, notify_email=True)
        notifications = Notification.objects.filter(user=self.pat.clariwebuser)
        assert len(notifications) == 2
        for notif in notifications:
            if notif.email:
                assert notif.email == self.pat.email
            elif notif.sms:
                assert(notif.sms ==
                       "{}".format(self.pat.clariwebuser.address_mobile_phone))
            assert notif.status == "SENT"

    def test_document_sms_notification_med(self):
        assert constance.config.SMS_NOTIFICATION_MEDECIN
        self.createDocument(self.med1, self.med1.to_actor(), 42,
                            notify_sms=True, notify_email=False)
        notifications = Notification.objects.filter(
            user=self.med1.clariwebuser)
        assert len(notifications) == 1
        assert "Demande" in notifications[0].origin

    def test_document_sms_notification_preleveur(self):
        assert constance.config.SMS_NOTIFICATION_PRELEVEUR
        self.createDocument(self.prel1, self.prel1.to_actor(), 42,
                            notify_sms=True, notify_email=False)
        notifications = Notification.objects.filter(
            user=self.prel1.clariwebuser)
        assert len(notifications) == 1
        assert "Demande" in notifications[0].origin

    def test_document_sms_notification_med_overdelay(self):
        assert constance.config.SMS_NOTIFICATION_MEDECIN
        doc = self.createDocument(self.med1, self.pat.to_actor(), 42,
                                  notify_sms=True, notify_email=False)
        Notification.objects.filter(user=self.med1.clariwebuser).delete()
        doc.date_add = timezone.now() - datetime.timedelta(days=60)
        doc.save()
        assert constance.config.NOTIFICATION_RESEND_MAXIMUM_DELAY < 60  # noqa
        notifications = Notification.objects.filter(
            user=self.med1.clariwebuser)
        assert len(notifications) == 0

    def test_document_sms_disable_notifications_patient(self):
        constance.config.SMS_NOTIFICATION_PATIENT = False
        constance.config.SMS_NOTIFICATION_MEDECIN = True
        constance.config.SMS_NOTIFICATION_CORRESPONDANT = True
        assert not constance.config.SMS_NOTIFICATION_PATIENT
        self.createDocument(self.pat, self.pat.to_actor(), 42,
                            notify_sms=True, notify_email=False)
        notifications = Notification.objects.filter(user=self.pat.clariwebuser)
        assert len(notifications) == 0

    def test_document_sms_disable_notifications_medecin(self):
        constance.config.SMS_NOTIFICATION_PATIENT = True
        constance.config.SMS_NOTIFICATION_MEDECIN = False
        constance.config.SMS_NOTIFICATION_CORRESPONDANT = True
        assert not constance.config.SMS_NOTIFICATION_MEDECIN
        self.createDocument(self.med1, self.pat.to_actor(), 42,
                            notify_sms=True, notify_email=False)
        notifications = Notification.objects.filter(
            user=self.med1.clariwebuser)
        assert len(notifications) == 0

    def test_document_sms_disable_notifications_preleveur(self):
        constance.config.SMS_NOTIFICATION_PRELEVEUR = False
        assert not constance.config.SMS_NOTIFICATION_PRELEVEUR
        self.createDocument(self.prel1, self.prel1.to_actor(), 42,
                            notify_sms=True, notify_email=False)
        notifications = Notification.objects.filter(
            user=self.prel1.clariwebuser)
        assert len(notifications) == 0

    def test_sms_notification_invalid_user(self):
        self.pat.email = ""
        self.pat.save()
        self.pat.clariwebuser.address_mobile_phone = ""
        self.pat.clariwebuser.save()
        self.createDocument(self.pat, self.pat.to_actor(), 42,
                            notify_sms=True, notify_email=True)
        notifications = Notification.objects.filter(user=self.pat.clariwebuser)
        assert len(notifications) == 0

    def test_disabled_user(self):
        self.pat.is_active = False
        self.pat.save(update_fields=('is_active', ))
        self.createDocument(self.pat, self.pat.to_actor(), 42,
                            notify_sms=True, notify_email=True)
        notifications = Notification.objects.filter(user=self.pat.clariwebuser)
        assert len(notifications) == 0

    def test_dontnotify_on_delete(self):
        self.pat.add_unconfirmed_email(self.pat.email)
        self.pat.confirm_email(self.pat.confirmation_key, self.pat.email)
        assert self.pat.is_confirmed
        doc = self.createDocument(self.pat, self.pat.to_actor(), 42,
                                  notify_sms=False, notify_email=True)
        notifications = Notification.objects.filter(user=self.pat.clariwebuser)
        notifications.delete()
        notifications = Notification.objects.filter(user=self.pat.clariwebuser)
        assert len(notifications) == 0
        doc.delete()
        notifications = Notification.objects.filter(user=self.pat.clariwebuser)
        assert len(notifications) == 0


class MergeUsersTestCase(ClariwebTestCaseBase, TestCase):
    def test_merge_debpt(self):
        self.pat3 = self.createClariwebUser("pat3", "PAT", 32, civilite="Mr")
        self.pat1.clariwebuser.debpt = 100
        self.pat1.clariwebuser.save()
        self.pat2.clariwebuser.debpt = 200
        self.pat2.clariwebuser.save()
        self.pat3.clariwebuser.debpt = 0
        self.pat3.clariwebuser.save()
        self.pat1.merge_foreignusers([self.pat2, self.pat3])
        pat1 = BasicUser.objects.get(username="pat1")
        assert pat1.clariwebuser.debpt == 300

    def test_merge_notification(self):
        Notification.objects.create(user=self.pat1.clariwebuser,
                                    status='SENT', email="test@test.com")
        Notification.objects.create(user=self.pat2.clariwebuser,
                                    status='ERROR', email="test@test.com")
        assert Notification.objects.filter(
            user=self.pat1.clariwebuser).count() == 1
        assert Notification.objects.filter(
            user=self.pat2.clariwebuser).count() == 1
        self.pat2.merge_foreignusers([self.pat1, ])
        assert Notification.objects.filter(
            user=self.pat2.clariwebuser).count() == 2

    def test_merge_accesslog(self):
        AccessLog.objects.create(user=self.pat1, message='AUTH_OK')
        AccessLog.objects.create(user=self.pat2, message='AUTH_BAD')
        assert AccessLog.objects.filter(user=self.pat1).count() == 1
        assert AccessLog.objects.filter(user=self.pat2).count() == 1
        self.pat1.merge_foreignusers([self.pat2, ])
        assert AccessLog.objects.filter(user=self.pat1).count() == 2

    def test_merge_documentaccesslog(self):
        doc1 = self.createDocument(self.pat1, self.pat1.to_actor(), 1)
        doc2 = self.createDocument(self.pat2, self.pat2.to_actor(), 2)
        DocumentAccessLog.objects.create(
            user=self.pat1.clariwebuser,
            document=doc1)
        DocumentAccessLog.objects.create(
            user=self.pat2.clariwebuser,
            document=doc2)
        assert DocumentAccessLog.objects.filter(
            user=self.pat1.clariwebuser).count() == 1
        assert DocumentAccessLog.objects.filter(
            user=self.pat2.clariwebuser).count() == 1
        self.pat2.merge_foreignusers([self.pat1, ])
        assert DocumentAccessLog.objects.filter(
            user=self.pat2.clariwebuser).count() == 2

    def test_merge_passwordreset(self):
        PasswordReset.objects.create(self.pat1, length=10)
        PasswordReset.objects.create(self.pat1)
        PasswordReset.objects.create(self.pat2)
        self.pat1.merge_foreignusers([self.pat2, ])
        assert len(PasswordReset.objects.filter(user__pk=self.pat1.pk)) == 1
        assert len(PasswordReset.objects.all()) == 1



class DocumentPerUserTestCase(ClariwebTestCaseBase, TestCase):
    def test_modify_usergroup(self):
        self.corr2.clariwebuser.group = "GRP1"
        self.corr2.clariwebuser.save()
        self.corr1.clariwebuser.group = "GRP1"
        self.corr1.clariwebuser.save()
        self.corr3 = self.createClariwebUser("cor3", "COR", 122, civilite="",
                                             group="GRP1")
        doc = Document(sil_document_pk=1,
                       sil_demande_pk=1,
                       sil_demande_number="1",
                       sil_demande_date=self.now,
                       content_type="application/pdf",
                       document="Compte-Rendu PDF",
                       datafile=File(io.StringIO("{}".format("test"))),
                       user=self.corr1.clariwebuser,
                       medecin=self.med2.to_actor(),
                       patient=self.pat1.to_actor()).save()
        doc2 = Document(sil_document_pk=2,
                       sil_demande_pk=2,
                       sil_demande_number="2",
                       sil_demande_date=self.now,
                       content_type="application/pdf",
                       document="Compte-Rendu PDF",
                       datafile=File(io.StringIO("{}".format("test"))),
                       user=self.corr2.clariwebuser,
                       medecin=self.med2.to_actor(),
                       patient=self.pat1.to_actor()).save()
        doc3 = Document(sil_document_pk=3,
                       sil_demande_pk=1,
                       sil_demande_number="3",
                       sil_demande_date=self.now,
                       content_type="application/pdf",
                       document="Compte-Rendu PDF",
                       datafile=File(io.StringIO("{}".format("test"))),
                       user=self.corr3.clariwebuser,
                       medecin=self.med2.to_actor(),
                       patient=self.pat1.to_actor()).save()
        assert len(Document.objects.foruser(self.pat1)) == 0
        assert len(Document.objects.foruser(self.pat2)) == 0
        assert len(Document.objects.foruser(self.med1)) == 0
        assert len(Document.objects.foruser(self.med2)) == 0
        assert len(Document.objects.foruser(self.med3)) == 0
        assert len(Document.objects.foruser(self.corr1)) == 3
        assert len(Document.objects.foruser(self.corr2)) == 3
        assert len(Document.objects.foruser(self.corr3)) == 3
        self.corr1.clariwebuser.group = "GRP2"
        self.corr1.clariwebuser.save()
        assert len(Document.objects.foruser(self.corr1)) == 1
        assert len(Document.objects.foruser(self.corr2)) == 2
        assert len(Document.objects.foruser(self.corr3)) == 2


    def test_modify_usergroupclinic(self):
        self.corr2.clariwebuser.group = ''
        self.corr2.clariwebuser.group_clinic = "GRP1"
        self.corr2.clariwebuser.save()
        self.corr1.clariwebuser.group = ''
        self.corr1.clariwebuser.group_clinic = "GRP1"
        self.corr1.clariwebuser.save()
        self.corr3 = self.createClariwebUser("cor3", "COR", 122, civilite="",
                                             group_clinic="GRP1")
        doc1 = Document(sil_document_pk=1,
                       sil_demande_pk=1,
                       sil_demande_number="1",
                       sil_demande_date=self.now,
                       content_type="application/pdf",
                       document="Compte-Rendu PDF",
                       datafile=File(io.StringIO("{}".format("test"))),
                       user=self.corr1.clariwebuser,
                       medecin=self.med2.to_actor(),
                       patient=self.pat1.to_actor())
        doc2 = Document(sil_document_pk=2,
                       sil_demande_pk=2,
                       sil_demande_number="2",
                       sil_demande_date=self.now,
                       content_type="application/pdf",
                       document="Compte-Rendu PDF",
                       datafile=File(io.StringIO("{}".format("test"))),
                       user=self.corr2.clariwebuser,
                       medecin=self.med2.to_actor(),
                       patient=self.pat1.to_actor())
        doc3 = Document(sil_document_pk=3,
                       sil_demande_pk=1,
                       sil_demande_number="3",
                       sil_demande_date=self.now,
                       content_type="application/pdf",
                       document="Compte-Rendu PDF",
                       datafile=File(io.StringIO("{}".format("test"))),
                       user=self.corr3.clariwebuser,
                       medecin=self.med2.to_actor(),
                       patient=self.pat1.to_actor())
        doc1.save()
        doc2.save()
        doc3.save()
        assert len(Document.objects.foruser(self.pat1)) == 0
        assert len(Document.objects.foruser(self.pat2)) == 0
        assert len(Document.objects.foruser(self.med1)) == 0
        assert len(Document.objects.foruser(self.med2)) == 0
        assert len(Document.objects.foruser(self.med3)) == 0
        assert len(Document.objects.foruser(self.corr1)) == 3
        assert len(Document.objects.foruser(self.corr2)) == 3
        assert len(Document.objects.foruser(self.corr3)) == 3
        self.corr1.clariwebuser.group_clinic = "GRP2"
        self.corr1.clariwebuser.save()
        assert len(Document.objects.foruser(self.corr1)) == 1
        assert len(Document.objects.foruser(self.corr2)) == 2
        assert len(Document.objects.foruser(self.corr3)) == 2
        doc1.user = self.corr2.clariwebuser
        doc1.save()
        assert len(Document.objects.foruser(self.corr1)) == 0
        assert len(Document.objects.foruser(self.corr2)) == 3
        assert len(Document.objects.foruser(self.corr3)) == 3
