# -*- coding: utf-8 -*-

from __future__ import absolute_import, unicode_literals

import os
import sys
import types
import datetime
import sentry_sdk
from sentry_sdk.integrations.django import DjangoIntegration
from pathlib import Path
from celery.schedules import crontab

BASE_DIR = Path(__file__).parent
TEST_MODE = bool(os.environ.get("TEST_MODE", False))
LOCAL_SETTINGS_PATH = Path(os.environ.get(
    "CLARIWEB_CONF", "~/.clariweb")).expanduser().absolute()

if not LOCAL_SETTINGS_PATH.exists() and not TEST_MODE:
    sys.stderr.write("{} does not exists. Can't continue\n".format(LOCAL_SETTINGS_PATH))
    sys.exit(1)

SECRET_KEY = 'r=yei+^_y3in)e68w!k%du_^r4q!uzkc#)q$z44=^8q5^5h1_7'
CLARIWEB_INSTALLED = False
DEBUG = True

ALLOWED_HOSTS = ["*", ]

CLARIWEB_APPS = [
    'clariweb.clariweb',
    'clariweb.backend',
    'clariweb.api',
]

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.postgres',
    'django_filters',
    'rest_framework',
    'django_extensions',
    'brake',
    'constance',
    'constance.backends.database',
    'immutable_unaccent',
    'simple_email_confirmation',
    'django_payzen',
] + CLARIWEB_APPS

PROJECT_APPS = CLARIWEB_APPS


MIDDLEWARE = (
    'clariweb.clariweb.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'clariweb.clariweb.middleware.RemoteAddrMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'clariweb.clariweb.middleware.NoIECacheMiddleware',
)

ROOT_URLCONF = 'clariweb.clariweb.urls'

WSGI_APPLICATION = 'clariweb.wsgi.application'

LANGUAGE_CODE = 'fr-fr'
TIME_ZONE = 'Europe/Paris'
USE_I18N = True
USE_L10N = True
USE_TZ = True

DOCUMENTS_ROOT = LOCAL_SETTINGS_PATH / 'documents'
LOG_DIR = LOCAL_SETTINGS_PATH / "logs"

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            str(LOCAL_SETTINGS_PATH / "templates"),
            str(BASE_DIR / "templates"),
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.contrib.auth.context_processors.auth',
            ],
        },
    },
]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': str(BASE_DIR / 'db.sqlite3'),
    }
}



LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'formatters': {
        'default': {
            'format': '%(levelname)s [%(asctime)s] %(message)s',
        }
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        },
        'logfile': {
            'class': 'logging.handlers.RotatingFileHandler',
            'filename': str(LOG_DIR / "clariweb.log"),
            'maxBytes': 5000 * 1024,
            'backupCount': 2,
            'formatter': 'default'
        },
        'celery': {
            'class': 'logging.handlers.RotatingFileHandler',
            'filename': str(LOG_DIR / "celery.log"),
            'maxBytes': 5000 * 1024,
            'backupCount': 2,
            'formatter': 'default'
        },
        'logdb': {
            'class': 'logging.handlers.RotatingFileHandler',
            'filename': str(LOG_DIR / "db.log"),
            'maxBytes': 5000 * 1024,
            'backupCount': 2,
            'formatter': 'default'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
        'django.db.backends': {
            'level': 'DEBUG',
            'handlers': ['logdb'],
        },
        'clariweb': {
            'level': 'DEBUG',
            'handlers': ['logfile'],
        },
        'backend': {
            'level': 'DEBUG',
            'handlers': ['logfile'],
        },
        'celery': {
            'handlers': ['celery'],
            'level': 'INFO',
            'propagate': True,
        },
        '': {
            'handlers': ['logfile'],
            'level': 'WARNING',
            'propagate': True,
        },
    }
}

REST_FRAMEWORK = {
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework.authentication.SessionAuthentication',
    ),
    'DEFAULT_PERMISSION_CLASSES': (
        'rest_framework.permissions.IsAuthenticated',
    ),
    'DEFAULT_RENDERER_CLASSES': (
        'rest_framework.renderers.JSONRenderer',
    ),
    'TEST_REQUEST_DEFAULT_FORMAT': 'json',
    'DATETIME_FORMAT': None,
    'DATE_FORMAT': None,
    'TIME_FORMAT': None,
    'UPLOADED_FILES_USE_URL': False,
}

ORPHANED_APPS_MEDIABASE_DIRS = {
    'clariweb': {
        'root': str(DOCUMENTS_ROOT),
    },
}

AUTH_USER_MODEL = "clariweb.BasicUser"
AUTHENTICATION_BACKENDS = ('clariweb.clariweb.auth.ClariwebAuthBackend', )

# AngularJS app
WEBUI_ROOT = str(BASE_DIR / 'webui' / 'dist')

REDIS_HOST = 'localhost'
REDIS_PORT = 6379
REDIS_DB = 0

CACHES = {
    "default": {
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": "redis://%s:%s/%s" % (REDIS_HOST, REDIS_PORT, REDIS_DB),
        "OPTIONS": {
            "CLIENT_CLASS": "django_redis.client.DefaultClient",
        }
    }
}

# Please don't override this.
TEST_RUNNER = 'clariweb.clariweb.testrunner.ClariwebTestRunner'

# Django-constance (settings in DB)
CONSTANCE_BACKEND = 'constance.backends.database.DatabaseBackend'
CONSTANCE_DATABASE_CACHE_BACKEND = 'default'

CONSTANCE_CONFIG = {
    "LOGIN_USING_SESSIONS": (
        True,
        "Utiliser les sessions pour les médecins / correspondants"),
    "UNDERAGE_FORBIDDEN": (
        True,
        "Interdit l'accès aux utilisateurs mineurs"),
    "EMAIL_SMS_CREDITREQUEST": (
        "commande@clarisys.fr",
        """Adresse E-Mail à utiliser pour envoyer les demandes d'achat de SMS"""),  # noqa
    "EMAIL_NOTIFICATION_UNCONFIRMED": (
        False,
        """Envoyer les notifications par email aux utilisateurs n'ayant par vérifié leur adresse email"""),  # noqa
    "CLARIWEB_ABSOLUTE_URL": (
        "http://resulabo.fr/?/app/",
        """Lien HTTP vers CLARIWEB (utilisé dans les notifications)"""),
    "SMS_LOGIN": (
        "",
        """Login du compte d'envoi SMS"""),
    "SMS_PASSWORD": (
        "",
        """Mot de passe du compte d'envoi SMS"""),
    "SMS_ACCOUNT": (
        "sms-zb40-1",
        """Compte SMS OVH (ne pas modifier!)"""),
    "SMS_NOTIFICATION_PATIENT": (
        False,
        """Activer les notifications par SMS pour les patients"""),
    "SMS_NOTIFICATION_MEDECIN": (
        False,
        """Activer les notifications par SMS pour les médecins"""),
    "SMS_NOTIFICATION_CORRESPONDANT": (
        False,
        """Activer les notifications par SMS pour les correspondants"""),
    "SMS_NOTIFICATION_PRELEVEUR": (
        False,
        """Activer les notifications par SMS pour les préleveurs"""),
    "NOTIFICATION_RESEND_MAXIMUM_DELAY": (
        30,
        """Durée en jours après laquelle une modification sur un document ne déclenchera plus de notifications"""),  # noqa
    "NOTIFICATION_SAMEUSER_MINIMUM_DELAY": (
        1,
        """Ne pas re-envoyer une notification à l'utilisateur si il en a déjà reçu une. Durée en heure(s) (0 = désactivé)"""),  # noqa
    "DEBPT_MINIMUM_AMOUNT": (
        0,
        """Montant en centimes d'euros avant le blocage de la consultation des résultats (0 = désactivé)"""),  # noqa
    "EMAIL_FROM": (
        "clariweb@resulabo.fr",
        """Adresse de provenance (FROM) des emails envoyés"""),
    "CUSTOM_TITLE": (
        """Serveur de résultats""",
        """Titre du site internet"""),
    "CUSTOM_LABO": (
        "Laboratoire CLARISYS Informatique",
        """Nom du laboratoire"""),
    "CUSTOM_ADDRESS_LINE1": (
        "31, rue des fleurs",
        """Adresse du laboratoire"""),
    "CUSTOM_ADDRESS_POSTALCODE": (
        "31000",
        """Code postal du laboratoire"""),
    "CUSTOM_ADDRESS_CITY": (
        "SAINT JEAN",
        """Ville du laboratoire"""),
    "CUSTOM_PHONE": (
        "09 72 11 43 60",
        """Numéro de téléphone du laboratoire"""),
    "CUSTOM_EMAIL": (
        "nopub@resulabo.fr",
        """Adresse EMAIL de la page de contact"""),
    "CUSTOM_WEBSITE": (
        "http://www.clarisys.fr/",
        """Adresse du site internet du laboratoire"""),
    "CUSTOM_SIGNATURE": (
        "",
        """Signature des emails de notification"""),
    "CUSTOM_LOGO": (
        "",
        """Adresse HTTP du logo du laboratoire"""),
    "CUSTOM_CONTACT_URL": (
        "",
        """Adresse HTTP personnalisée pour remplacer la page contact de ClariWeb"""),  # noqa
    "CUSTOM_IFRAME": (
        "",
        """Encart affiché dans la page contact"""),
    "CUSTOM_THEME_KEY": (
        "",
        """Style sélectionné pour tous les utilisateurs. Ne touchez pas ce paramètre si vous n'êtes pas sûr"""),  # noqa
    "CUSTOM_PAYMENTS_ENABLED": (
        False,
        """Activer les paiements en ligne"""),
    "SECURITY_IP_ADDRESSES_CLINIC": (
        [u"0.0.0.0/0"],
        """Liste d'adresses IP autorisées à utiliser les fonctionnalités spécifiques aux cliniques"""),  # noqa
    "CLINIC_SHARE_SERVICES": (
        True,
        """Permettre aux services clinique de partager leurs résultats (un service voit les résultats d'un autre service)"""),  # noqa
    "CLINIC_ENABLED": (
        False,
        """Activer la consultation clinique (nécessite CLARILAB 3 R14 ou ultérieur)"""),
    "SECURITY_IP_ADDRESSES_ADMIN": (
        ["176.31.136.155/32"],
        """Liste d'adresses IP autorisées à accéder au compte administrateur (exemples 192.168.0.0/24 10.0.0.1). (176.31.136.155/32 est utilisé par CLARISYS pour la maintenance)"""),  # noqa
}

# simple email simple_email_confirmation
# We want to do this ourselve;
SIMPLE_EMAIL_CONFIRMATION_AUTO_ADD = False
SIMPLE_EMAIL_CONFIRMATION_PERIOD = datetime.timedelta(days=30)

# Must override thoses settings in local_settings.
DEFAULT_FROM_EMAIL = "clariweb@resulabo.fr"
SERVER_EMAIL = DEFAULT_FROM_EMAIL
EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'
USE_X_FORWARDED_HOST = False
# LC: If we use cache backend for session, we will run into problems
# of session dropper because cache is full. We don't want that
SESSION_ENGINE = 'django.contrib.sessions.backends.db'
# OVH API SMS
SMS_OVH_HTTP2SMS = "https://www.ovh.com/cgi-bin/sms/http2sms.cgi"
# This setting must be valid validated from source
# in the OVH interface
SMS_OVH_HTTP2SMS_FROM = "RESULABOFR"
# How much days we want to retain the user session active
# (seulement pour les médecins, correspondants, ..)
# in seconds
SESSION_COOKIE_AGE = 300
SESSION_LOGIN_EXPIRY_CHOICES = {
    0: 300,  # 5 minutes
    1: 600,  # 10 minutes for payment
    2: 604800,  # 1 week
    3: 7257600,  # 3 months
}

# Age minimum pour la consultation
UNDERAGE = 18

# Celery settings
CELERY_BROKER_URL = 'redis://%s:%s/%s' % (REDIS_HOST, REDIS_PORT, REDIS_DB)
CELERY_RESULT_BACKEND = 'redis://%s:%s/%s' % (REDIS_HOST, REDIS_PORT, REDIS_DB)
CELERY_TASK_SERIALIZER = 'json'
CELERY_RESULT_SERIALIZER = 'json'
CELERY_ACCEPT_CONTENT = ['json']
CELERY_ENABLE_UTC = True
CELERY_RESULT_EXPIRES = 300
# 24 hours timeout for redis backend to avoid timeouting our "retry" jobs
CELERY_BROKER_TRANSPORT_OPTIONS = {'visibility_timeout': 3600 * 24}

# RPC REST api for clarilab
API_RPC_TOKEN = "QVkBcLeMRRy0TVUjYGFUopfIWkuyschv4nSb7QSuXzWlmDBtefYYnPVCzqrjCc3I"  # noqa

CELERY_BEAT_SCHEDULE = {
    'sync_sms_quota': {
        'task': 'clariweb.backend.tasks.update_sms_quota',
        'schedule': crontab(hour=4, minute=10),
    },
    'cleanpasswordreset': {
        'task': 'clariweb.backend.tasks.periodic_clear_passwordreset',
        'schedule': crontab(hour=4, minute=0),
    },
    'deleteorphaned': {
        'task': 'clariweb.backend.tasks.periodic_deleteorphaned',
        'schedule': crontab(day_of_week='sunday', hour=1, minute=0),
    },
    'refreshstatsviews': {
        'task': 'clariweb.backend.tasks.periodic_refreshstatsviews',
        'schedule': crontab(hour=5, minute=0),
    },
    'clean_paymentrequests': {
        'task': 'clariweb.backend.tasks.periodoc_clean_paymentrequests',
        'schedule': crontab(hour="*"),
    },
    'clearsessions': {
        'task': 'clariweb.backend.tasks.periodic_cleansessions',
        'schedule': crontab(hour=5, minute=30),
    },
}

# Payzen
VADS_CTX_MODE = "TEST"
VADS_SITE_ID = 59095860
VADS_CERTIFICATE = "2514867257925641"
VADS_CURRENCY = 978  # Euros
VADS_ACTION_MODE = "INTERACTIVE"
VADS_CONTRIB = 'clariweb'
PAYZEN_REQUEST_URL = 'https://demo.payzen.eu/vads-payment/'

CELERY_TIMEZONE = TIME_ZONE
CLARILAB_API_URL = "http://localhost:8000/api"
CLARILAB_API_TOKEN = "ahhahH3hoh3ahwah4Oog2Eyoh0Iengesie7loo3ce2jeeciemai4he8bahpukahj"  # noqa

sentry_sdk.init(
    dsn="https://00b54e90ac67450887a3265ccad62acf:96236089e4a24ae0afe162732e0d1bf2@sentry.clarisys.fr/23",
    integrations=[DjangoIntegration()]
)

def load_settings(filename, settings, silent=False):
    conf = types.ModuleType('temp_config')
    conf.__file__ = filename
    try:
        with open(filename) as source_file:
            exec(source_file.read(), conf.__dict__)
    except IOError as e:
        import errno
        if silent and e.errno in (errno.ENOENT, errno.EISDIR):
            return settings
        e.strerror = 'Unable to load configuration file (%s)' % e.strerror
        raise
    for setting in dir(conf):
        if not setting.isupper():
            continue
        setting_value = getattr(conf, setting)
        setattr(settings, setting, setting_value)


if not TEST_MODE:
    load_settings(str(LOCAL_SETTINGS_PATH / "clariweb.conf.py"), sys.modules[__name__])
    if not DOCUMENTS_ROOT.exists():
        DOCUMENTS_ROOT.mkdir(0o700)
    if not LOG_DIR.exists():
        LOG_DIR.mkdir()

# Python 3.5 fix/workaround for os.path.join
BASE_DIR = str(BASE_DIR)
DOCUMENTS_ROOT = str(DOCUMENTS_ROOT)
LOG_DIR = str(LOG_DIR)

if not TEST_MODE and not CLARIWEB_INSTALLED:  # noqa
    sys.stderr.write("Clariweb is not properly configured (CLARIWEB_INSTALLED = False)\n")
    sys.exit(1)
