import os
import sys


def convert_options_to_env(options):
    for k, v in options.items():
        if v is None:
            continue
        key = 'UWSGI_' + k.upper().replace('-', '_')
        if isinstance(v, (str, int)):
            value = str(v)
        elif v is True:
            value = 'true'
        elif v is False:
            value = 'false'
        else:
            raise TypeError('Unknown option type: %r (%s)' % (k, type(v)))
        yield key, value


class ClariwebHTTPServer(object):
    name = 'http'

    def __init__(
        self, debug=False, workers=3, validate=True, extra_options=None
    ):
        options = {}
        if extra_options is not None:
            for k, v in extra_options.items():
                options[str(k)] = str(v)
        options.setdefault('auto-procname', True)
        options.setdefault('buffer-size', 32768)
        options.setdefault('die-on-term', True)
        options.setdefault('disable-logging', False)
        options.setdefault('disable-write-exception', True)
        options.setdefault('http-timeout', 30)
        options.setdefault('ignore-sigpipe', True)
        options.setdefault('ignore-write-errors', True)
        options.setdefault('limit-post', 20971520)
        options.setdefault('log-x-forwarded-for', False)
        options.setdefault('memory-report', True)
        options.setdefault('module', 'clariweb.wsgi:application')
        options.setdefault('need-app', True)
        options.setdefault('post-buffering', 65536)
        options.setdefault('procname-prefix-spaced', '[clariweb]')
        options.setdefault('protocol', 'http')
        #options.setdefault('reload-on-rss', 600)
        options.setdefault('threads', 4)
        options.setdefault('thunder-lock', True)
        options.setdefault('vacuum', True)
        options.setdefault('virtualenv', sys.prefix)
        options.setdefault('workers', workers)
        options.setdefault('chmod-socket', "666")
        options.setdefault(
            'log-format',
            '%(addr) - [%(ltime)] "%(method) %(uri) %(proto)" %(status) %(size) "%(referer)" "%(uagent)"'
        )
        # Required arguments that should not be overridden
        options['master'] = True
        options['enable-threads'] = True
        options['lazy-apps'] = True
        options['single-interpreter'] = True
        self.options = options

    def validate_settings(self):
        from django.conf import settings as django_settings
        del(django_settings)

    def prepare_environment(self, env=None):
        if env is None:
            env = os.environ

        # Move all of the options into UWSGI_ env vars
        for k, v in convert_options_to_env(self.options):
            env.setdefault(k, v)
        # Look up the bin directory where `clariweb` exists, which should be
        # sys.argv[0], then inject that to the front of our PATH so we can reliably
        # find the `uwsgi` that's installed when inside virtualenv.
        # This is so the virtualenv doesn't need to be sourced in, which effectively
        # does exactly this.
        virtualenv_path = os.path.dirname(os.path.abspath(sys.argv[0]))
        current_path = env.get('PATH', '')
        if virtualenv_path not in current_path:
            env['PATH'] = '%s:%s' % (virtualenv_path, current_path)

    def run(self):
        self.prepare_environment()
        os.execvp('uwsgi', ['uwsgi'])
